CREATE OR REPLACE PACKAGE     PKG_COMUN AS

ErrorParlamentoNoExiste EXCEPTION;
ErrorParlamentarioNoExiste EXCEPTION;
ErrorPartidoNoExiste EXCEPTION;
ErrorParlamentarioParlamento EXCEPTION;
ErrorParlamentarioNoParlamento EXCEPTION;
ErrorParlamentarioPartido EXCEPTION;
ErrorParlamentarioNoPartido EXCEPTION;
ErrorEjercicioNoExiste EXCEPTION;
ErrorCategoriaGastoNoExiste EXCEPTION;
ErrorTipoIVANoExiste EXCEPTION;
ErrorProveedorNoExiste EXCEPTION;
ErrorEstadoNoExiste EXCEPTION;
ErrorFormaPagoNoExiste EXCEPTION;
ErrorGastoNoExiste EXCEPTION;
ErrorEstadisticaNoExiste EXCEPTION;

/**
Registra la ejecucin de un procedimiento, sus parmetros de entrada y sus parmetros de salida.

Inserta una fila (entrada) en la tabla Registro. 

En cada entrada de la tabla Registro quedan registrados:
- El nombre del procedimiento ejecutado.
- El nombre del usuario que invoc la ejecucin del procedimiento.
- El instante de tiempo en que se produjo la ejecucin del procedimiento.
- La lista de parmetros de entrada y sus valores.
- La lista de parmetros de salida y sus valores.

@param p_procedimiento                  Nombre del procedimiento ejecutado.
@param p_usuario                        Nombre del usuario que invoc el procedimiento.
@param p_instante                       Instante de tiempo en que fue ejecutado el procedimiento.
@param p_entrada                        Parmetros de entrada y sus valores.
@param p_salida                         Parmetros de salida y sus valores.
*/
PROCEDURE entrada_registro (
  p_procedimiento IN Registro.PROCEDIMIENTO % TYPE,
  p_usuario       IN Registro.USUARIO % TYPE,
  p_instante      IN Registro.INSTANTE % TYPE,
  p_entrada       IN Registro.ENTRADA % TYPE,
  p_salida        IN Registro.SALIDA % TYPE);

/**
Dado un identificador de Parlamento, comprueba si ese Parlamento existe en la base de datos.

@param p_idParlamento                   Identificador del Parlamento a buscar en la base de datos.
	
@throws ErrorParlamentoNoExiste         Si el Parlamento buscado no existe en la base de datos.
*/
PROCEDURE existe_parlamento (
  p_idParlamento IN Parlamento.idParlamento % TYPE);

/**	
Dado un identificador de Parlamentario, comprueba si ese Parlamentario existe en la base de datos.
	
@param p_idParlamentario                Identificador del Parlamentario a buscar en la base de datos.

@throws ErrorParlamentarioNoExiste      Si el Parlamentario buscado no existe en la base de datos.
*/
PROCEDURE existe_parlamentario (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE);

/**
Dado un identificador de partido, comprueba si ese partido existe en la base de datos.
	
@param p_idPartido                      Identificador del partido a buscar en la base de datos.
	
@throws ErrorParlamentarioNoExiste      Si el Parlamentario buscado no existe en la base de datos.	
*/
PROCEDURE existe_partido (
  p_idPartido IN Partido.idPartido % TYPE);

/**
Dado un identificador de Parlamentario comprueba que ese Parlamentario no est activo ya en algn Parlamento.

La comprobacin a realizar consiste en buscar la existencia de alguna fila en la tabla ParlamentoDH que
corresponda a ese Parlamentario y cuya fecha de finalizacin sea NULL.
 
La existencia de esa fila provocar el lanzamiento de la excepcin ErrorParlamentarioParlamento.

@param p_idParlamentario                Identificador del Parlamentario que debe comprobarse.

@throws ErrorParlamentarioParlamento    Si el Parlamentario ya est activo en algn Parlamento.
*/
PROCEDURE parlamentario_no_en_parlamento (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE);

/**
Dado un identificador de Parlamentario, comprueba que ese Parlamentario est activo en algn Parlamento.

La comprobacin a realizar consiste en buscar la existencia de alguna fila en la tabla ParlamentoDH que
corresponda a ese Parlamentario y cuya fecha de finalizacin sea NULL.

La no existencia de esa fila provocar el lanzamiento de la excepcin ErrorParlamentarioNoParlamento.

@param p_idParlamentario                Identificador del Parlamentario que debe comprobarse.

@throws ErrorParlamentarioNoParlamento  Si el Parlamentario no est activo en ningn Parlamento.
*/
PROCEDURE parlamentario_en_parlamento (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE);

/**
Dados un identificador de Parlamento, un identificador de Parlamentario y una fecha, comprueba si ese Parlamentario 
estaba activo en ese Parlamento en esa fecha.

La comprobacin a realizar consiste en buscar la existencia de alguna fila en la tabla ParlamentoDH que
corresponda a ese Parlamento y a ese Parlamentario y cuya fecha de inicio sea anterior o igual a la fecha
buscada, y a la vez la fecha buscada sea anterior o igual a la fecha de finalizacin o esta ltima sea NULL.

La no existencia de esa fila provocar el lanzamiento de la excepcin ErrorParlamentarioNoParlamento.

@param p_idParlamento                   Identificador del Parlamento que debe comprobarse.
@param p_idParlamentario                Identificador del Parlamentario que debe comprobarse.
@param p_fecha                          Fecha que debe comprobarse.

@throws ErrorParlamentarioNoParlamento	Si el Parlamentario no estaba activo en ese Parlamento en esa fecha.
*/
PROCEDURE parlamentario_en_parlamento (
  p_idParlamento    IN Parlamento.idParlamento % TYPE,
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE,
  p_fecha           IN ParlamentoDH.desde % TYPE);

/**
Dado un identificador de Parlamentario, comprueba que ese Parlamentario no est activo ya en algn partido.

La comprobacin a realizar consiste en buscar la existencia de alguna fila en la tabla PartidoDH que
corresponda a ese Parlamentario y cuya fecha de finalizacin sea NULL.

La existencia de esa fila provocar el lanzamiento de la excepcin ErrorParlamentarioPartido.

@param p_idParlamentario                Identificador del Parlamentario que debe comprobarse.

@throws ErrorParlamentarioPartido       Si el Parlamentario ya est activo en algn partido.
*/
PROCEDURE parlamentario_no_en_partido (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE);

/**
Dado un identificador de Parlamentario, comprueba que ese Parlamentario est activo en algn partido.

La comprobacin a realizar consiste en buscar la existencia de alguna fila en la tabla PartidoDH que
corresponda a ese Parlamentario y cuya fecha de finalizacin sea NULL.

La no existencia de esa fila provocar el lanzamiento de la excepcin ErrorParlamentarioNoPartido.

@param p_idParlamentario                Identificador del Parlamentario que debe comprobarse.

@throws ErrorParlamentarioNoPartido     Si el Parlamentario no est activo en ningn partido.
*/
PROCEDURE parlamentario_en_partido (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE);

/**
Dado un identificador de Parlamentario y una fecha, comprueba si ese Parlamentario estaba activo en algn
partido en esa fecha.

La comprobacin a realizar consiste en buscar la existencia de alguna fila en la tabla PartidoDH que
corresponda a ese Parlamentario y cuya fecha de inicio sea anterior o igual a la fecha buscada, y a la vez la fecha
buscada sea anterior o igual a la fecha de finalizacin o esta ltima sea NULL.

La no existencia de esa fila provocar el lanzamiento de la excepcin ErrorParlamentarioNoParlamento.	

@param p_idParlamentario                Identificador del Parlamentario que debe comprobarse.
@param p_fecha                          Fecha que debe comprobarse.

@throws ErrorParlamentarioNoPartido     Si el Parlamentario no est activo en ningn partido en esa fecha.
*/
PROCEDURE parlamentario_en_partido (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE,
  p_fecha           IN PartidoDH.desde % TYPE);

/**
Dada una fecha, comprueba que exista un ejercicio en la base de datos al que pertenezca esa fecha.

La comprobacin a realizar consiste en buscar la existencia de alguna fila en la tabla Ejercicio cuya fecha de 
inicio sea anterior o igual a la fecha buscada y cuya fecha de finalizacin sea igual o posterior a la fecha
buscada.

La no existencia de esa fila provocar el lanzamiento de la excepcin ErrorEjercicioNoExiste.

@param p_fecha                          Fecha cuyo ejercicio hay que buscar en la base de datos.

@throws ErrorEjercicioNoExiste          Si no existe el ejercicio al que pertenece esa fecha en la base de datos.	
*/
PROCEDURE existe_ejercicio (
  p_fecha IN Ejercicio.inicio % TYPE);

/**
Dado un identificador de ejercicio, comprueba que ese ejercicio exista en la base de datos.

@param p_idEjercicio                    Identificador del ejercicio a buscar en la base de datos.

@throws ErrorEjercicioNoExiste          Si el ejercicio buscado no existe en la base de datos.	
*/
PROCEDURE existe_ejercicio (
  p_idEjercicio IN Ejercicio.idEjercicio % TYPE);

/**
Dado un identificador de categora de gasto, comprueba que esa categora de gasto exista en la base de datos.

@param p_idCategoriaGasto               Identificador de la categora de gasto a buscar en la base de datos.

@throws ErrorCategoriaGastoNoExiste     Si la categora de gasto buscada no existe en la base de datos.		
*/
PROCEDURE existe_categoriaGasto (
  p_idCategoriaGasto IN CategoriaGasto.idCategoriaGasto % TYPE);

/**
Dado un identificador de tipo de IVA, comprueba que ese tipo de IVA exista en la base de datos.

@param p_idTipoIVA                      Identificador del tipo de IVA a buscar en la base de datos.

@throws ErrorTipoIVAnoExiste            Si el tipo de IVA buscado no existe en la base de datos.			
*/
PROCEDURE existe_tipoIVA (
  p_idTipoIVA IN TipoIVA.idTipoIVA % TYPE);

/**
Dado un identificador de proveedor, comprueba que ese proveedor exista en la base de datos.

@param p_idProveedor                    Identificador del proveedor a buscar en la base de datos.

@throws ErrorProveedorNoExiste          Si el proveedor buscado no existe en la base de datos.				
*/
PROCEDURE existe_proveedor (
  p_idProveedor IN Proveedor.idProveedor % TYPE);

/**
Dado un identificador de estado (de un gasto), comprueba que ese estado exista en la base de datos.

@param p_idEstado                       Identificador del estado a buscar en la base de datos.

@throws ErrorEstadoNoExiste             Si el estado buscado no existe en la base de datos.	
*/
PROCEDURE existe_estado (
  p_idEstado IN Estado.idEstado % TYPE);

/**
Dado un identificador de forma de pago, comprueba que esa forma de pago exista en la base de datos.

@param p_idFormaPago                    Identificador de la forma de pago a buscar en la base de datos.

@throws ErrorFormaPagoNoExiste          Si la forma de pago buscada no existe en la base de datos.	
*/
PROCEDURE existe_formaPago (
  p_idFormaPago IN FormaPago.idFormaPago % TYPE);

/**
Dado un identificador de gasto, comprueba que ese gasto exista en la base de datos.

@param p_idGasto                        Identificador del gasto a buscar en la base de datos.

@throws ErrorGastoNoExiste              Si el gasto buscado no existe en la base de datos.	
*/
PROCEDURE existe_gasto (
  p_idGasto IN Gasto.idGasto % TYPE);

/**
Dado un identificador de ejercicio y un identificador de Parlamento, comprueba si la estadstica del Parlamento
para ese ejercicio exista en la base de datos.

@param p_idEstado                       Identificador del ejercicio que debe comprobarse.
@param p_idParlamento                   Identificador del Parlamento que debe comprobarse.

@throws ErrorEstadisticaNoExiste        Si la estadstica de ese Parlamento en ese ejercicio no existe en la base de datos.		
*/
PROCEDURE existe_parlamentoST (
  p_idEjercicio  IN ParlamentoST.idEjercicio % TYPE,
  p_idParlamento IN ParlamentoST.idParlamento % TYPE);

/**
Dado un identificador de ejercicio, comprueba si existen estadsticas de Parlamentos para ese Ejercicio. 

@param idEjercicio                      Identificador del ejercicio que debe comprobarse.

@throws ErrorEstadisticaNoExiste        Si no existen estadsticas de Parlamento para ese Ejercicio.
*/
PROCEDURE existe_parlamentoST (
  p_idEjercicio IN ParlamentoST.idEjercicio % TYPE);

/**
Dado un identificador de ejercicio, un identificador de Parlamento y un identificador de Parlamentario, comprueba 
si existe la estadstica de ese Parlamentario para ese ejercicio y ese Parlamento.

@param idEjercicio                      Identificador del ejercicio que debe comprobarse.
@param idParlamento                     Identificador del Parlamento que debe comprobarse.
@param idParlamentario                  Identificador del Parlamentario que debe comprobarse.

@throws ErrorEstadisticaNoExiste        Si no existen estadsticas para ese Parlamentario en ese ejercicio en ese Parlamento.
*/
PROCEDURE existe_parlamentarioST (
  p_idEjercicio     IN ParlamentarioST.idEjercicio % TYPE,
  p_idParlamento    IN ParlamentarioST.idParlamento % TYPE,
  p_idParlamentario IN ParlamentarioST.idParlamentario % TYPE);

/**
Dados un identificador de ejercicio y un identificador de Parlamentario, comprueba si existen estadsticas de
Parlamentario para ese ejercicio en algn Parlamento.

@param idEjercicio                      Identificador del ejercicio que debe comprobarse.
@param idParlamentario                  Identificador del Parlamentario que debe comprobarse.

@throws ErrorEstadisticaNoExiste        Si no existen estadsticas para ese Parlamentario en ese Ejercicio.
*/
PROCEDURE existe_parlamentarioST (
  p_idEjercicio     IN ParlamentarioST.idEjercicio % TYPE,
  p_idParlamentario IN ParlamentarioST.idParlamentario % TYPE);

/**
Dado un identificador de ejercicio, comprueba si existen estadsticas de Parlamentario para ese Ejercicio.

@param idEjercicio                      Identificador del ejercicio que debe comprobarse.

@throws ErrorEstadisticaNoExiste        Si no existen estadsticas de Parlamentario para ese Ejercicio.
*/
PROCEDURE existe_parlamentarioST (
  p_idEjercicio IN ParlamentarioST.idEjercicio % TYPE);

END PKG_COMUN;
/


CREATE OR REPLACE PACKAGE BODY     PKG_COMUN AS

--
-- Registra la ejecucin de un procedimiento, sus parmetros de entrada y sus parmetros de salida.
--
PROCEDURE entrada_registro (
  p_procedimiento IN Registro.PROCEDIMIENTO % TYPE,
  p_usuario       IN Registro.USUARIO % TYPE,
  p_instante      IN Registro.INSTANTE % TYPE,
  p_entrada       IN Registro.ENTRADA % TYPE,
  p_salida        IN Registro.SALIDA % TYPE)
AS
BEGIN
  INSERT INTO REGISTRO
      (PROCEDIMIENTO,
       USUARIO,
       INSTANTE,
       ENTRADA,
       SALIDA)
    VALUES
      (p_procedimiento,
       p_usuario,
       p_instante,
       p_entrada,
       p_salida);
END entrada_registro;

--
-- Dado un identificador de Parlamento, comprueba si ese Parlamento existe en la base de datos.
--
PROCEDURE existe_parlamento (
  p_idParlamento IN Parlamento.idParlamento % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idParlamento IS NULL)
  THEN
    RAISE ErrorParlamentoNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      Parlamento
    WHERE
      idParlamento = p_idParlamento;

    IF (existe = 0)
    THEN
      RAISE ErrorParlamentoNoExiste;
    END IF;
  END IF;
END existe_parlamento;

--
-- Dado un identificador de Parlamentario, comprueba si ese Parlamentario existe en la base de datos.
--
PROCEDURE existe_parlamentario (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idParlamentario IS NULL)
  THEN
    RAISE ErrorParlamentarioNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      Parlamentario
    WHERE
      idParlamentario = p_idParlamentario;

    IF (existe = 0)
    THEN
      RAISE ErrorParlamentarioNoExiste;
    END IF;
  END IF;
END existe_parlamentario;

--
-- Dado un identificador de partido, comprueba si ese partido existe en la base de datos.
--
PROCEDURE existe_partido (
  p_idPartido IN Partido.idPartido % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idPartido IS NULL)
  THEN
    RAISE ErrorPartidoNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      PARTIDO
    WHERE
      idPartido = p_idPartido;

    IF (existe = 0)
    THEN
      RAISE ErrorPartidoNoExiste;
    END IF;
  END IF;
END existe_partido;

--
-- Dado un identificador de Parlamentario comprueba que ese Parlamentario no est activo ya en algn Parlamento.
--
PROCEDURE parlamentario_no_en_parlamento (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE)
AS
  existe NATURAL;
BEGIN
  SELECT
    COUNT (*)
  INTO
    existe
  FROM
    ParlamentoDH
  WHERE
    idParlamentario = p_idParlamentario
    AND
    hasta IS NULL;

  IF (existe <> 0)
  THEN
    RAISE ErrorParlamentarioParlamento;
  END IF;
END parlamentario_no_en_parlamento;

--
-- Dado un identificador de Parlamentario, comprueba que ese Parlamentario est activo en algn Parlamento.
--
PROCEDURE parlamentario_en_parlamento (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idParlamentario IS NULL)
  THEN
    RAISE ErrorParlamentarioNoParlamento;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      ParlamentoDH
    WHERE
      idParlamentario = p_idParlamentario
      AND
      hasta IS NULL;

    IF (existe = 0)
    THEN
      RAISE ErrorParlamentarioNoParlamento;
    END IF;
  END IF;
END parlamentario_en_parlamento;

--
-- Dados un identificador de Parlamento, un identificador de Parlamentario y una fecha, comprueba si ese Parlamentario estaba activo en ese Parlamento en esa fecha.
--
PROCEDURE parlamentario_en_parlamento (
  p_idParlamento    IN Parlamento.idParlamento % TYPE,
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE,
  p_fecha           IN ParlamentoDH.desde % TYPE)
AS
  existe NATURAL;
BEGIN
  SELECT
    COUNT (*)
  INTO
    existe
  FROM
    ParlamentoDH
  WHERE
    idParlamento = p_idParlamento
    AND
    idParlamentario = p_idParlamentario
    AND
    desde <= p_fecha
    AND
    (
    p_fecha <= hasta
      OR
      hasta IS NULL
      );

  IF (existe = 0)
  THEN
    RAISE ErrorParlamentarioNoParlamento;
  END IF;
END parlamentario_en_parlamento;

--
-- Dado un identificador de Parlamentario, comprueba que ese Parlamentario no est activo ya en algn partido.
--
PROCEDURE parlamentario_no_en_partido (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE)
AS
  existe NATURAL;
BEGIN
  SELECT
    COUNT (*)
  INTO
    existe
  FROM
    PartidoDH
  WHERE
    idParlamentario = p_idParlamentario
    AND
    hasta IS NULL;

  IF (existe <> 0)
  THEN
    RAISE ErrorParlamentarioPartido;
  END IF;
END parlamentario_no_en_partido;

--
-- Dado un identificador de Parlamentario, comprueba que ese Parlamentario est activo en algn partido.
--
PROCEDURE parlamentario_en_partido (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE)
AS
  existe NATURAL;
BEGIN
  SELECT
    COUNT (*)
  INTO
    existe
  FROM
    PartidoDH
  WHERE
    idParlamentario = p_idParlamentario
    AND
    hasta IS NULL;

  IF (existe = 0)
  THEN
    RAISE ErrorParlamentarioNoPartido;
  END IF;
END parlamentario_en_partido;

--
-- Dado un identificador de Parlamentario y una fecha, comprueba si ese Parlamentario estaba activo en algn partido en esa fecha.
--
PROCEDURE parlamentario_en_partido (
  p_idParlamentario IN Parlamentario.idParlamentario % TYPE,
  p_fecha           IN PartidoDH.desde % TYPE)
AS
  existe NATURAL;
BEGIN
  SELECT
    COUNT (*)
  INTO
    existe
  FROM
    PartidoDH
  WHERE
    idParlamentario = p_idParlamentario
    AND
    desde <= p_fecha
    AND
    (
    p_fecha <= hasta
      OR
      hasta IS NULL
      );

  IF (existe = 0)
  THEN
    RAISE ErrorParlamentarioNoPartido;
  END IF;
END parlamentario_en_partido;

--
-- Dada una fecha, comprueba que exista un ejercicio en la base de datos al que pertenezca esa fecha.
--
PROCEDURE existe_ejercicio (
  p_fecha IN Ejercicio.inicio % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_fecha IS NULL)
  THEN
    RAISE ErrorEjercicioNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      Ejercicio
    WHERE
      inicio <= p_fecha
      AND
      p_fecha <= fin;

    IF (existe = 0)
    THEN
      RAISE ErrorEjercicioNoExiste;
    END IF;
  END IF;
END existe_ejercicio;

--
-- Dado un identificador de ejercicio, comprueba que ese ejercicio exista en la base de datos.
--
PROCEDURE existe_ejercicio (
  p_idEjercicio IN Ejercicio.idEjercicio % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idEjercicio IS NULL)
  THEN
    RAISE ErrorEjercicioNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      Ejercicio
    WHERE
      idEjercicio = p_idEjercicio;

    IF (existe = 0)
    THEN
      RAISE ErrorEjercicioNoExiste;
    END IF;

  END IF;
END existe_ejercicio;

--
-- Dado un identificador de categora de gasto, comprueba que esa categora de gasto exista en la base de datos.
--
PROCEDURE existe_categoriaGasto (
  p_idCategoriaGasto IN CategoriaGasto.idCategoriaGasto % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idCategoriaGasto IS NULL)
  THEN
    RAISE ErrorCategoriaGastoNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      CategoriaGasto
    WHERE
      idCategoriaGasto = p_idCategoriaGasto;

    IF (existe = 0)
    THEN
      RAISE ErrorCategoriaGastoNoExiste;
    END IF;
  END IF;
END existe_categoriaGasto;

--
-- Dado un identificador de tipo de IVA, comprueba que ese tipo de IVA exista en la base de datos.
--
PROCEDURE existe_tipoIVA (
  p_idTipoIVA IN TipoIVA.idTipoIVA % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idTipoIVA IS NULL)
  THEN
    RAISE ErrorTipoIVANoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      TipoIVA
    WHERE
      idTipoIVA = p_idTipoIVA;

    IF (existe = 0)
    THEN
      RAISE ErrorTipoIVANoExiste;
    END IF;
  END IF;
END existe_tipoIVA;

--
-- Dado un identificador de proveedor, comprueba que ese proveedor exista en la base de datos.
--
PROCEDURE existe_proveedor (
  p_idProveedor IN Proveedor.idProveedor % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idProveedor IS NULL)
  THEN
    RAISE ErrorProveedorNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      Proveedor
    WHERE
      idProveedor = p_idProveedor;

    IF (existe = 0)
    THEN
      RAISE ErrorProveedorNoExiste;
    END IF;
  END IF;
END existe_proveedor;

--
-- Dado un identificador de estado (de un gasto), comprueba que ese estado exista en la base de datos.
--
PROCEDURE existe_estado (
  p_idEstado IN Estado.idEstado % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idEstado IS NULL)
  THEN
    RAISE ErrorEstadoNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      Estado
    WHERE
      idEstado = p_idEstado;

    IF (existe = 0)
    THEN
      RAISE ErrorEstadoNoExiste;
    END IF;
  END IF;
END existe_estado;

--
-- Dado un identificador de forma de pago, comprueba que esa forma de pago exista en la base de datos.
--
PROCEDURE existe_formaPago (
  p_idFormaPago IN FormaPago.idFormaPago % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idFormaPago IS NULL)
  THEN
    RAISE ErrorFormaPagoNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      FormaPago
    WHERE
      idFormaPago = p_idFormaPago;

    IF (existe = 0)
    THEN
      RAISE ErrorFormaPagoNoExiste;
    END IF;
  END IF;
END existe_formaPago;

--
-- Dado un identificador de gasto, comprueba que ese gasto exista en la base de datos.
--
PROCEDURE existe_gasto (
  p_idGasto IN Gasto.idGasto % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idGasto IS NULL)
  THEN
    RAISE ErrorGastoNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      Gasto
    WHERE
      idGasto = p_idGasto;

    IF (existe = 0)
    THEN
      RAISE ErrorGastoNoExiste;
    END IF;
  END IF;
END existe_gasto;

--
-- Dado un identificador de ejercicio y un identificador de Parlamento, comprueba si la estadstica del Parlamento para ese ejercicio exista en la base de datos.
--
PROCEDURE existe_parlamentoST (
  p_idEjercicio  IN ParlamentoST.idEjercicio % TYPE,
  p_idParlamento IN ParlamentoST.idParlamento % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idEjercicio IS NULL
      OR
      p_idParlamento IS NULL)
  THEN
    RAISE ErrorEstadisticaNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      ParlamentoST
    WHERE
      idEjercicio = p_idEjercicio
      AND
      idParlamento = p_idParlamento;

    IF (existe = 0)
    THEN
      RAISE ErrorEstadisticaNoExiste;
    END IF;
  END IF;
END existe_parlamentoST;

--
-- Dado un identificador de ejercicio, comprueba si existen estadsticas de Parlamentos para ese Ejercicio. 
--
PROCEDURE existe_parlamentoST (
  p_idEjercicio IN ParlamentoST.idEjercicio % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idEjercicio IS NULL)
  THEN
    RAISE ErrorEstadisticaNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      ParlamentoST
    WHERE
      idEjercicio = p_idEjercicio;

    IF (existe = 0)
    THEN
      RAISE ErrorEstadisticaNoExiste;
    END IF;
  END IF;
END existe_parlamentoST;

--
-- Dado un identificador de ejercicio, un identificador de Parlamento y un identificador de Parlamentario, comprueba 
-- si existe la estadstica de ese Parlamentario para ese ejercicio y ese Parlamento.
--
PROCEDURE existe_parlamentarioST (
  p_idEjercicio     IN ParlamentarioST.idEjercicio % TYPE,
  p_idParlamento    IN ParlamentarioST.idParlamento % TYPE,
  p_idParlamentario IN ParlamentarioST.idParlamentario % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idEjercicio IS NULL
      OR
      p_idParlamento IS NULL
      OR
      p_idParlamentario IS NULL)
  THEN
    RAISE ErrorEstadisticaNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      ParlamentarioST
    WHERE
      idEjercicio = p_idEjercicio
      AND
      idParlamento = p_idParlamento
      AND
      idParlamentario = p_idParlamentario;

    IF (existe = 0)
    THEN
      RAISE ErrorEstadisticaNoExiste;
    END IF;
  END IF;
END existe_parlamentarioST;

--
-- Dados un identificador de ejercicio y un identificador de Parlamentario, comprueba si existen estadsticas de
-- Parlamentario para ese ejercicio en algn Parlamento.
--
PROCEDURE existe_parlamentarioST (
  p_idEjercicio     IN ParlamentarioST.idEjercicio % TYPE,
  p_idParlamentario IN ParlamentarioST.idParlamentario % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idEjercicio IS NULL
      OR
      p_idParlamentario IS NULL)
  THEN
    RAISE ErrorEstadisticaNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      ParlamentarioST
    WHERE
      idEjercicio = p_idEjercicio
      AND
      idParlamentario = p_idParlamentario;

    IF (existe = 0)
    THEN
      RAISE ErrorEstadisticaNoExiste;
    END IF;
  END IF;
END existe_parlamentarioST;

--
-- Dado un identificador de ejercicio, comprueba si existen estadsticas de Parlamentario para ese Ejercicio.
--
PROCEDURE existe_parlamentarioST (
  p_idEjercicio IN ParlamentarioST.idEjercicio % TYPE)
AS
  existe NATURAL;
BEGIN
  IF (p_idEjercicio IS NULL)
  THEN
    RAISE ErrorEstadisticaNoExiste;
  ELSE
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      ParlamentarioST
    WHERE
      idEjercicio = p_idEjercicio;

    IF (existe = 0)
    THEN
      RAISE ErrorEstadisticaNoExiste;
    END IF;
  END IF;
END existe_parlamentarioST;

END PKG_COMUN;
/
