CREATE OR REPLACE PACKAGE     pkg_gasto AS

ErrorFecha EXCEPTION;
ErrorEjercicioCerrado EXCEPTION;
ErrorParlamentario EXCEPTION;
ErrorNoParlamentario EXCEPTION;
ErrorBaseImponible EXCEPTION;
ErrorBaseImponibleNegativa EXCEPTION;
ErrorProveedor EXCEPTION;
ErrorFormaPago EXCEPTION;
ErrorNoFormaPago EXCEPTION;
ErrorPagado EXCEPTION;

/**
Procedimiento para dar de alta un gasto en la base de datos.

Al insertar una fila en la base de datos, el atributo idGasto se asigna automticamente mediante una secuencia.

@param p_fecha                        Fecha en la que se ha producido el gasto.
@param p_idParlamento                 Identificador del parlamento al que se imputa el gasto.
@param p_idParlamentario              Identificador del parlamentario al que se imputa el gasto (si lo hay).
@param p_baseImponible                Base imponible del importe del gasto.
@param p_idTipoIVA                    Identificador del tipo de IVA aplicable al gasto.
@param p_idCategoriaGasto             Identificador de la categora del gasto.
@param p_idEstado                     Identificador del estado del gasto.
@param p_idFormaPago                  Identificador de la forma de pago del gasto (si ha sido pagado).
@param p_idProveedor                  Identificador del proveedor del bien o servicio (si lo hay).
@param p_descripcion                  Texto descriptivo del gasto.
@param p_justificacion                Texto justificativo del gasto.
@param p_presupuestoURL               URL al documento que contiene el presupuesto del gasto (si lo hay).
@param p_facturaURL                   URL al documento que contiene la factura del gasto (si la hay).
@param p_contratoURL                  URL al documento que contiene el contrato del gasto (si lo hay).
@param p_rsp                          Resultado de la ejecucin del procedimiento.

@throws ErrorFecha                    Si se especifica una fecha NULL para el gasto.
@throws ErrorEjercicioCerrado         Si se especifica una fecha perteneciente a un ejercicio cerrado.
@throws ErrorParlamentario            Si se especifica un idParlamentario NULL cuando no est permitido.
@throws ErrorNoParlamentario          Si se especifica un idParlamentario cuando no est permitido.
@throws ErrorBaseImponible            Si se especifica una base imponible NULL.
@throws ErrorBaseImponibleNegativa    Si se especifica una base imponible negativa para el gasto.
@throws ErrorProveedor                Si se especifica un idProveedor NULL cuando no est permitido.
@throws ErrorFormaPago                Si se especifica un idFormaPago NULL cuando no est permitido.
@throws ErrorNoFormaPago              Si se especifica un idFormapago cuando no est permitido.
*/
PROCEDURE alta (
  p_fecha            IN  Gasto.fecha % TYPE,
  p_idParlamento     IN  Gasto.idParlamento % TYPE,
  p_idParlamentario  IN  Gasto.idParlamentario % TYPE,
  p_baseImponible    IN  Gasto.baseImponible % TYPE,
  p_idTipoIVA        IN  Gasto.idTipoIVA % TYPE,
  p_idCategoriaGasto IN  Gasto.idCategoriaGasto % TYPE,
  p_idEstado         IN  Gasto.idEstado % TYPE,
  p_idFormaPago      IN  Gasto.idFormaPago % TYPE,
  p_idProveedor      IN  Gasto.idProveedor % TYPE,
  p_descripcion      IN  Gasto.descripcion % TYPE,
  p_justificacion    IN  Gasto.justificacion % TYPE,
  p_presupuestoURL   IN  Gasto.presupuestoURL % TYPE,
  p_facturaURL       IN  Gasto.facturaURL % TYPE,
  p_contratoURL      IN  Gasto.contratoURL % TYPE,
  p_rsp              OUT VARCHAR2);

/**
Procedimiento para dar de baja un gasto en la base de datos.

@param p_idGasto                      Identificador del gasto a dar de baja.
@param p_rsp                          Resultado de la ejecucin del procedimiento.

@throws ErrorPagado                   Si el gasto que se intenta dar de baja est pagado.
@throws ErrorEjercicioCerrado         Si el gasto que se intenta dar de baja pertenece a un ejercicio cerrado.
*/
PROCEDURE baja (
  p_idGasto IN  Gasto.idGasto % TYPE,
  p_rsp     OUT VARCHAR2);

/**
Procedimiento para modificar un gasto en la base de datos.

@param p_idGasto                      Identificador del gasto a modificar.
@param p_fecha                        Fecha en la que se ha producido el gasto.
@param p_idParlamento                 Identificador del parlamento al que se imputa el gasto.
@param p_idParlamentario              Identificador del parlamentario al que se imputa el gasto (si lo hay).
@param p_baseImponible                Base imponible del importe del gasto.
@param p_idTipoIVA                    Identificador del tipo de IVA aplicable al gasto.
@param p_idCategoriaGasto             Identificador de la categora del gasto.
@param p_idEstado                     Identificador del estado del gasto.
@param p_idFormaPago                  Identificador de la forma de pago del gasto (si ha sido pagado).
@param p_idProveedor                  Identificador del proveedor del bien o servicio (si lo hay).
@param p_descripcion                  Texto descriptivo del gasto.
@param p_justificacion                Texto justificativo del gasto.
@param p_presupuestoURL               URL al documento que contiene el presupuesto del gasto (si lo hay).
@param p_facturaURL                   URL al documento que contiene la factura del gasto (si la hay).
@param p_contratoURL                  URL al documento que contiene el contrato del gasto (si lo hay).
@param p_rsp                          Resultado de la ejecucin del procedimiento.

@throws ErrorPagado                   Si el gasto que se intenta modificar est pagado.
@throws ErrorEjercicioCerrado         Si se especifica una fecha perteneciente a un ejercicio cerrado.
@throws ErrorFecha                    Si se especifica una fecha NULL para el gasto.
@throws ErrorParlamentario            Si se especifica un idParlamentario NULL cuando no est permitido.
@throws ErrorNoParlamentario          Si se especifica un idParlamentario cuando no est permitido.
@throws ErrorBaseImponible            Si se especifica una base imponible NULL.
@throws ErrorBaseImponibleNegativa    Si se especifica una base imponible negativa para el gasto.
@throws ErrorProveedor                Si se especifica un idProveedor NULL cuando no est permitido.
@throws ErrorFormaPago                Si se especifica un idFormaPago NULL cuando no est permitido.
@throws ErrorNoFormaPago              Si se especifica un idFormapago cuando no est permitido.
*/
PROCEDURE modificacion (
  p_idGasto          IN  Gasto.idGasto % TYPE,
  p_fecha            IN  Gasto.fecha % TYPE,
  p_idParlamento     IN  Gasto.idParlamento % TYPE,
  p_idParlamentario  IN  Gasto.idParlamentario % TYPE,
  p_baseImponible    IN  Gasto.baseImponible % TYPE,
  p_idTipoIVA        IN  Gasto.idTipoIVA % TYPE,
  p_idCategoriaGasto IN  Gasto.idCategoriaGasto % TYPE,
  p_idEstado         IN  Gasto.idEstado % TYPE,
  p_idFormaPago      IN  Gasto.idFormaPago % TYPE,
  p_idProveedor      IN  Gasto.idProveedor % TYPE,
  p_descripcion      IN  Gasto.descripcion % TYPE,
  p_justificacion    IN  Gasto.justificacion % TYPE,
  p_presupuestoURL   IN  Gasto.presupuestoURL % TYPE,
  p_facturaURL       IN  Gasto.facturaURL % TYPE,
  p_contratoURL      IN  Gasto.contratoURL % TYPE,
  p_rsp              OUT VARCHAR2);

/**
Procedimiento para cerrar un ejercicio.

Una vez cerrado un ejercicio ya no se pueden dar de alta nuevos gastos en ese ejercicio, ni dar de baja
ni modificar los gastos imputados en ese ejercicio.

@param p_idEjercicio                  Identificador del ejercicio a cerrar.
@param p_rsp                          Resultado de la ejecucin del procedimiento.
*/
PROCEDURE cerrar_ejercicio (
  p_idEjercicio IN  Gasto.idEjercicio % TYPE,
  p_rsp         OUT VARCHAR2);

END pkg_gasto;
/


CREATE OR REPLACE PACKAGE BODY     pkg_gasto AS

entrada Registro.entrada % TYPE;	-- String con los parmetros de entrada y sus valores.
salida Registro.salida % TYPE;		-- String con los parmetros de salida y sus valores.

RequiereProveedor CONSTANT Gasto.baseImponible % TYPE := 30.00;		-- Base imponible a partir del cual es obligatorio indicar un proveedor.

/**
Dados un identificador de ejercicio, un identificador de parlamento y un importe, actualiza la columna TotalGastosGenerales de la estadstica 
ParlamentoST correspondiente.

Este procedimiento es de uso interno dentro del paquete pkg_gasto.

@param p_idEjercicio        Identificador del ejercicio de la estadstica a actualizar.
@param p_idParlamento       Identificador del parlamento de la estadstica a actualizar.
@param p_importeGasto       Importe a actualizar en la estadstica.
*/
PROCEDURE estadistica_ParlamentoST (
  p_idEjercicio  IN Gasto.idEjercicio % TYPE,
  p_idParlamento IN Gasto.idParlamento % TYPE,
  p_importeGasto IN Gasto.baseImponible % TYPE);

/**
Dados un identificador de ejercicio, un identificador de parlamento, un identificador de parlamentario y un importe, actualiza las columnas
TotalGastosParlamentarios, NumParlamentarios, MinGastoParlamentario y MaxGastoParlamentario de la estadstica ParlamentoST correspondiente.

Este procedimiento es de uso interno dentro del paquete pkg_gasto.

@param p_idEjercicio        Identificador del ejercicio de la estadstica a actualizar.
@param p_idParlamento       Identificador del parlamento de la estadstica a actualizar.
@param p_idParlamentario    Identificador del parlamentario de la estadstica a actualizar.
@param p_importeGasto       Importe a actualizar en la estadstica.
*/
PROCEDURE estadistica_ParlamentoST (
  p_idEjercicio     IN Gasto.idEjercicio % TYPE,
  p_idParlamento    IN Gasto.idParlamento % TYPE,
  p_idParlamentario IN Gasto.idParlamentario % TYPE,
  p_importeGasto    IN Gasto.baseImponible % TYPE);

/**
Dados un identificador de ejercicio, un identificador de parlamento, un identificador de parlamentario y un importe, actualiza la columna
TotalGastos de la estadstica ParlamentarioST correspondiente.

Este procedimiento es de uso interno dentro del paquete pkg_gasto.

@param p_idEjercicio        Identificador del ejercicio de la estadstica a actualizar.
@param p_idParlamento       Identificador del parlamento de la estadstica a actualizar.
@param p_idParlamentario    Identificador del parlamentario de la estadstica a actualizar.
@param p_importeGasto       Importe a actualizar en la estadstica.	
*/
PROCEDURE estadistica_ParlamentarioST (
  p_idEjercicio     IN Gasto.idEjercicio % TYPE,
  p_idParlamento    IN Gasto.idParlamento % TYPE,
  p_idParlamentario IN Gasto.idParlamentario % TYPE,
  p_importeGasto    IN Gasto.baseImponible % TYPE);

--
-- Procedimiento para dar de alta un gasto en la base de datos.
--
PROCEDURE alta (
  p_fecha            IN  Gasto.fecha % TYPE,
  p_idParlamento     IN  Gasto.idParlamento % TYPE,
  p_idParlamentario  IN  Gasto.idParlamentario % TYPE,
  p_baseImponible    IN  Gasto.baseImponible % TYPE,
  p_idTipoIVA        IN  Gasto.idTipoIVA % TYPE,
  p_idCategoriaGasto IN  Gasto.idCategoriaGasto % TYPE,
  p_idEstado         IN  Gasto.idEstado % TYPE,
  p_idFormaPago      IN  Gasto.idFormaPago % TYPE,
  p_idProveedor      IN  Gasto.idProveedor % TYPE,
  p_descripcion      IN  Gasto.descripcion % TYPE,
  p_justificacion    IN  Gasto.justificacion % TYPE,
  p_presupuestoURL   IN  Gasto.presupuestoURL % TYPE,
  p_facturaURL       IN  Gasto.facturaURL % TYPE,
  p_contratoURL      IN  Gasto.contratoURL % TYPE,
  p_rsp              OUT VARCHAR2)
AS
  x_idEjercicio     Ejercicio.idEjercicio % TYPE;
  x_inicio          Ejercicio.inicio % TYPE;
  x_fin             Ejercicio.fin % TYPE;
  x_cerradoSN       Ejercicio.cerradoSN % TYPE;
  x_ParlamentarioSN TipoGasto.ParlamentarioSN % TYPE;
  x_idPartido       Partido.idPartido % TYPE;
  x_formaPagoSN     Estado.FormaPagoSN % TYPE;
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Inicializa los campos que calcula este procedimiento.
    x_idEjercicio := NULL;
    x_idPartido := NULL;

    -- Controla la fecha del Gasto.
    IF (p_fecha IS NULL)
    THEN
      -- Debe especificarse una fecha vlida para el Gasto.
      RAISE ErrorFecha;
    END IF;

    -- Controla que la fecha est entre el rango de fechas de algn Ejercicio dado de alta en la base de datos.
    pkg_comun.existe_ejercicio (p_fecha);
    -- Consulta el Ejercicio al que pertenece la fecha del Gasto.
    SELECT
      idEjercicio,
      inicio,
      fin,
      cerradoSN
    INTO
      x_idEjercicio,
      x_inicio,
      x_fin,
      x_cerradoSN
    FROM
      Ejercicio
    WHERE
      inicio <= p_fecha
      AND
      p_fecha <= fin;

    -- Controla que el Ejercicio al que pertenece la fecha del Gasto no est cerrado.
    IF (x_cerradoSN = 'T')
    THEN
      RAISE ErrorEjercicioCerrado;
    END IF;

    -- Controla que el Parlamento al que se imputar el Gasto exista en la base de datos.
    pkg_comun.existe_parlamento (p_idParlamento);

    -- Controla que la CategoriaGasto exista en la base de datos.
    pkg_comun.existe_categoriaGasto (p_idCategoriaGasto);

    -- Consulta la CategoriaGasto y su TipoGasto.
    SELECT
      ParlamentarioSN
    INTO
      x_ParlamentarioSN
    FROM
      CategoriaGasto,
      TipoGasto
    WHERE
      CategoriaGasto.idCategoriaGasto = p_idCategoriaGasto
      AND
      TipoGasto.idTipoGasto = CategoriaGasto.idTipoGasto;

    -- Controla si la CategoriaGasto y su TipoGasto requieren que el Gasto sea asignado a un Parlamentario.
    IF (x_ParlamentarioSN = 'T')
    THEN
      -- La CategoriaGasto requiere que el Gasto sea asignado a un Parlamentario.
      IF (p_idParlamentario IS NULL)
      THEN
        RAISE ErrorParlamentario;
      ELSE -- Comprobaciones asociadas al Parlamentario.
        -- Controla que el Parlamentario exista en la base de datos.
        pkg_comun.existe_parlamentario (p_idParlamentario);
        -- Controla que el Parlamentario estuviera en este Parlamento en la fecha en que se produjo el Gasto.
        pkg_comun.parlamentario_en_parlamento (p_idParlamento, p_idParlamentario, p_fecha);
        -- Comprueba que el Parlamentario estuviera en algn Partido en la fecha en que se produjo el Gasto.
        -- Se considera que para poder imputar un Gasto a un Parlamentario en un Parlamento este debe estar en un Partido.
        -- Si no milita en ningn Partido entonces ha de pertenecer a un Partido ficticio llamado GRUPO MIXTO.
        pkg_comun.parlamentario_en_partido (p_idParlamentario, p_fecha);
        -- Consulta ese perodo (PartidoDH) para obtener el identificador del Partido.
        SELECT
          idPartido
        INTO
          x_idPartido
        FROM
          PartidoDH
        WHERE
          idParlamentario = p_idParlamentario
          AND
          desde <= p_fecha
          AND
          (p_fecha <= hasta
            OR
            hasta IS NULL);
      END IF; -- Comprobaciones asociadas al Parlamentario.
    ELSE
      -- La CategoriaGasto no permite que el Gasto sea asignado a un Parlamentario.
      IF (p_idParlamentario IS NOT NULL)
      THEN
        RAISE ErrorNoParlamentario;
      END IF;
    END IF;

    -- Controla la baseImponible del Gasto.
    IF (p_baseImponible IS NULL)
    THEN
      RAISE ErrorBaseImponible;
    ELSE
      IF (p_baseImponible < 0.00)
      THEN
        RAISE ErrorBaseImponibleNegativa;
      END IF;
    END IF;

    -- Controla que el TipoIVA exista en la base de datos.
    pkg_comun.existe_tipoIVA (p_idTipoIVA);

    -- Controla el Proveedor.
    IF (p_idProveedor IS NULL)
    THEN
      IF (p_baseImponible >= RequiereProveedor)
      THEN
        -- Se considera que el Proveedor es OBLIGATORIO si la base imponible del gasto es mayor o igual a RequiereProveedor.
        RAISE ErrorProveedor;
      END IF;
    ELSE
      -- Se considera que el Proveedor es OPCIONAL si la base imponible del gasto es menor que RequiereProveedor.
      -- Si se especifica Proveedor controla su existencia en la base de datos.
      pkg_comun.existe_PROVEEDOR (p_idProveedor);
    END IF;

    -- Controla que el Estado del Gasto exista en la base de datos.
    pkg_comun.existe_ESTADO (p_idEstado);
    -- Consulta el Estado del Gasto.
    SELECT
      FormaPagoSN
    INTO
      x_formaPagoSN
    FROM
      Estado
    WHERE
      idEstado = p_idEstado;

    -- Controla si el Estado del Gasto requiere que se especifique una FormaPago.
    IF (x_formaPagoSN = 'T')
    THEN
      -- El Estado del Gasto requiere una FormaPago.
      IF (p_idFormaPago IS NULL)
      THEN
        RAISE ErrorFormaPago;
      ELSE
        -- Controla que la FormaPago exista en la base de datos.
        pkg_comun.existe_FORMAPAGO (p_idFormaPago);
      END IF;
    ELSE
      -- El Estado del Gasto no permite una FormaPago.
      IF (p_idFormaPago IS NOT NULL)
      THEN
        RAISE ErrorNoFormaPago;
      END IF;
    END IF;

    -- No se realizan comprobaciones sobre los parmetros: p_descripcion, p_justificacion, p_presupuestoURL, p_facturaURL, p_contratoURL.

    -- Actualizacin Estadsticas.
    IF (x_ParlamentarioSN = 'T')
    THEN
      -- Si el Gasto es imputable a un Parlamentario entonces se suma la base imponible del gasto
      -- en la tabla de estadstica ParlamentarioST que a su vez se suma en la tabla de estadstica ParlamentoST.
      estadistica_ParlamentarioST (x_idEjercicio, p_idParlamento, p_idParlamentario, p_baseImponible);
    ELSE
      -- Si el Gasto no es imputable a un Parlamentario entonces se suma la base imponible del gasto
      -- en la tabla de estadstica ParlamentoST.
      estadistica_ParlamentoST (x_idEjercicio, p_idParlamento, p_baseImponible);
    END IF;

    -- Realiza una insercin en la tabla Gasto.
    INSERT INTO Gasto
        (fecha,
         idEjercicio,
         idParlamento,
         idParlamentario,
         idPartido,
         descripcion,
         baseImponible,
         idTipoIVA,
         idCategoriaGasto,
         idEstado,
         idFormaPago,
         idProveedor,
         justificacion,
         presupuestoURL,
         facturaURL,
         contratoURL)
      VALUES
        (p_fecha,
         x_idEjercicio,
         p_idParlamento,
         p_idParlamentario,
         x_idPartido,
         p_descripcion,
         p_baseImponible,
         p_idTipoIVA,
         p_idCategoriaGasto,
         p_idEstado,
         p_idFormaPago,
         p_idProveedor,
         p_justificacion,
         p_presupuestoURL,
         p_facturaURL,
         p_contratoURL);

    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN ErrorFecha THEN
      p_rsp := 'ERROR: Debe especificarse una fecha vlida para el gasto.';
    WHEN ErrorEjercicioCerrado THEN
      p_rsp := 'ERROR: El ejercicio [' || x_idEjercicio || '] desde [' || x_inicio || '] hasta [' || x_fin || '] est cerrado.';
    WHEN ErrorParlamentario THEN
      p_rsp := 'ERROR: La categora de gasto [' || p_idCategoriaGasto || '] requiere que el gasto sea asignado a un parlamentario.';
    WHEN ErrorNoParlamentario THEN
      p_rsp := 'ERROR: La categora de gasto [' || p_idCategoriaGasto || '] no permite que el gasto sea asignado a un parlamentario.';
    WHEN ErrorBaseImponible THEN
      p_rsp := 'ERROR: Debe especificarse la base imponible del gasto.';
    WHEN ErrorBaseImponibleNegativa THEN
      p_rsp := 'ERROR: La base imponible del gasto no puede ser negativa.';
    WHEN ErrorProveedor THEN
      p_rsp := 'ERROR: Debe especificarse un proveedor obligatoriamente por ser un gasto superior a [' || RequiereProveedor || '] euros.';
    WHEN ErrorFormaPago THEN
      p_rsp := 'ERROR: El estado [' || p_idEstado || '] requiere que se especifique una forma de pago en el gasto.';
    WHEN ErrorNoFormaPago THEN
      p_rsp := 'ERROR: El estado [' || p_idEstado || '] no permite que se especifique una forma de pago en el gasto.';
    WHEN pkg_comun.ErrorEjercicioNoExiste THEN
      p_rsp := 'ERROR: La fecha [' || p_fecha || '] no pertenece a ningn ejercicio.';
    WHEN pkg_comun.ErrorParlamentoNoExiste THEN
      p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
    WHEN pkg_comun.ErrorCategoriaGastoNoExiste THEN
      p_rsp := 'ERROR: La categora de gasto [' || p_idCategoriaGasto || '] no existe.';
    WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
    WHEN pkg_comun.ErrorParlamentarioNoParlamento THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no estaba en el parlamento [' || p_idParlamento || '] en fecha [' || p_fecha || '].';
    WHEN pkg_comun.ErrorParlamentarioNoPartido THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no estaba en ningn partido en fecha [' || p_fecha || '].';
    WHEN pkg_comun.ErrorProveedorNoExiste THEN
      p_rsp := 'ERROR: El proveedor [' || p_idProveedor || '] no existe.';
    WHEN pkg_comun.ErrorTipoIVANoExiste THEN
      p_rsp := 'ERROR: El tipo de IVA [' || p_idTipoIVA || '] no existe.';
    WHEN pkg_comun.ErrorEstadoNoExiste THEN
      p_rsp := 'ERROR: El estado [' || p_idEstado || '] no existe.';
    WHEN pkg_comun.ErrorFormaPagoNoExiste THEN
      p_rsp := 'ERROR: La forma de pago [' || p_idFormaPago || '] no existe.';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'fecha = [' || p_fecha || '] + idParlamento = [' || p_idParlamento || '] + idParlamentario = [' || p_idParlamentario || '] + baseImponible = [' || p_baseImponible || '] + idTipoIVA = [' || p_idTipoIVA || '] + idCategoriaGasto = [' || p_idCategoriaGasto || '] + idEstado = [' || p_idEstado || '] + idFormaPago = [' || p_idFormaPago || '] + idProveedor = [' || p_idProveedor || '] + descripcion = [' || p_descripcion || '] + justificacion = [' || p_justificacion || '] + presupuestoURL = [' || p_presupuestoURL || '] + facturaURL = [' || p_facturaURL || '] + contratoURL = [' || p_contratoURL || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_gasto.alta', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END alta;

--
-- Procedimiento para dar de baja un gasto en la base de datos.
--
PROCEDURE baja (
  p_idGasto IN  Gasto.idGasto % TYPE,
  p_rsp     OUT VARCHAR2)
AS
  r_idEjercicio     Gasto.idEjercicio % TYPE;
  r_idParlamento    Gasto.idParlamento % TYPE;
  r_idParlamentario Gasto.idParlamentario % TYPE;
  r_idFormaPago     Gasto.idFormaPago % TYPE;
  r_baseImponible   Gasto.baseImponible % TYPE;
  r_idTipoIVA       Gasto.idTipoIVA % TYPE;

  x_inicio          Ejercicio.inicio % TYPE;
  x_fin             Ejercicio.fin % TYPE;
  x_cerradoSN       Ejercicio.cerradoSN % TYPE;
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el Gasto a dar de baja exista en la base de datos.
    pkg_comun.existe_gasto (p_idGasto);
    -- Consulta el Gasto de la base de datos.
    SELECT
      idEjercicio,
      idParlamento,
      idParlamentario,
      idFormaPago,
      baseImponible,
      idTipoIVA
    INTO
      r_idEjercicio,
      r_idParlamento,
      r_idParlamentario,
      r_idFormaPago,
      r_baseImponible,
      r_idTipoIVA
    FROM
      Gasto
    WHERE
      idGasto = p_idGasto;

    -- Controla que el Gasto no est pagado.
    IF (r_idFormaPago IS NOT NULL)
    THEN
      RAISE ErrorPagado;
    END IF;

    -- Consulta el Ejercicio al que pertenece el Gasto.
    SELECT
      inicio,
      fin,
      cerradoSN
    INTO
      x_inicio,
      x_fin,
      x_cerradoSN
    FROM
      Ejercicio
    WHERE
      idEjercicio = r_idEjercicio;
    -- Controla que el Ejercicio no est cerrado.
    IF (x_cerradoSN = 'T')
    THEN
      RAISE ErrorEjercicioCerrado;
    END IF;

    -- Actualiza Estadstica.
    IF (r_idParlamentario IS NOT NULL)
    THEN
      -- Si el Gasto es imputable a un Parlamentario entonces se resta la base imponible del gasto
      -- en la tabla de estadstica ParlamentarioST que a su vez se resta en la tabla de estadstica ParlamentoST.
      estadistica_ParlamentarioST (r_idEjercicio, r_idParlamento, r_idParlamentario, -r_baseImponible);
    ELSE
      -- Si el Gasto no es imputable a un Parlamentario entonces se resta la base imponible del gasto
      -- en la tabla de estadstica ParlamentoST.
      estadistica_ParlamentoST (r_idEjercicio, r_idParlamento, -r_baseImponible);
    END IF;

    -- Realiza un borrado en la tabla Gasto.
    DELETE FROM
        Gasto
      WHERE
        idGasto = p_idGasto;

    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN pkg_comun.ErrorGastoNoExiste THEN
      p_rsp := 'ERROR: El gasto [' || p_idGasto || '] no existe.';
    WHEN ErrorPagado THEN
      p_rsp := 'ERROR: El gasto [' || p_idGasto || '] no puede eliminarse porque est pagado.';
    WHEN ErrorEjercicioCerrado THEN
      p_rsp := 'ERROR: El ejercicio [' || r_idEjercicio || '] desde [' || x_inicio || '] hasta [' || x_fin || '] est cerrado.';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idGasto = [' || p_idGasto || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_gasto.baja', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END baja;

--
-- Procedimiento para modificar un gasto en la base de datos.
--
PROCEDURE modificacion (
  p_idGasto          IN  Gasto.idGasto % TYPE,
  p_fecha            IN  Gasto.fecha % TYPE,
  p_idParlamento     IN  Gasto.idParlamento % TYPE,
  p_idParlamentario  IN  Gasto.idParlamentario % TYPE,
  p_baseImponible    IN  Gasto.baseImponible % TYPE,
  p_idTipoIVA        IN  Gasto.idTipoIVA % TYPE,
  p_idCategoriaGasto IN  Gasto.idCategoriaGasto % TYPE,
  p_idEstado         IN  Gasto.idEstado % TYPE,
  p_idFormaPago      IN  Gasto.idFormaPago % TYPE,
  p_idProveedor      IN  Gasto.idProveedor % TYPE,
  p_descripcion      IN  Gasto.descripcion % TYPE,
  p_justificacion    IN  Gasto.justificacion % TYPE,
  p_presupuestoURL   IN  Gasto.presupuestoURL % TYPE,
  p_facturaURL       IN  Gasto.facturaURL % TYPE,
  p_contratoURL      IN  Gasto.contratoURL % TYPE,
  p_rsp              OUT VARCHAR2)
AS
  r_fecha               Gasto.fecha % TYPE;
  r_idEjercicio         Gasto.idEjercicio % TYPE;
  r_idParlamento        Gasto.idParlamento % TYPE;
  r_idParlamentario     Gasto.idParlamentario % TYPE;
  r_idPartido           Gasto.idPartido % TYPE;
  r_baseImponible       Gasto.baseImponible % TYPE;
  r_idTipoIVA           Gasto.idTipoIVA % TYPE;
  r_idCategoriaGasto    Gasto.idCategoriaGasto % TYPE;
  r_idEstado            Gasto.idEstado % TYPE;
  r_idFormaPago         Gasto.idFormaPago % TYPE;
  r_idProveedor         Gasto.idProveedor % TYPE;
  r_descripcion         Gasto.descripcion % TYPE;
  r_justificacion       Gasto.justificacion % TYPE;
  r_presupuestoURL      Gasto.presupuestoURL % TYPE;
  r_facturaURL          Gasto.facturaURL % TYPE;
  r_contratoURL         Gasto.contratoURL % TYPE;

  idEjercicioCambia     BOOLEAN := FALSE;
  idParlamentoCambia    BOOLEAN := FALSE;
  idParlamentarioCambia BOOLEAN := FALSE;
  importeGastoCambia    BOOLEAN := FALSE;

  x_idEjercicio         Ejercicio.idEjercicio % TYPE;
  x_inicio              Ejercicio.inicio % TYPE;
  x_fin                 Ejercicio.fin % TYPE;
  x_cerradoSN           Ejercicio.cerradoSN % TYPE;
  x_idTipoGasto         CategoriaGasto.idTipoGasto % TYPE;
  x_ParlamentarioSN     TipoGasto.ParlamentarioSN % TYPE;
  x_idPartido           Partido.idPartido % TYPE;
  x_formaPagoSN         Estado.FormaPagoSN % TYPE;
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el Gasto a dar de baja exista en la base de datos.
    pkg_comun.existe_gasto (p_idGasto);
    -- Consulta el Gasto de la base de datos.
    SELECT
      fecha,
      idEjercicio,
      idParlamento,
      idParlamentario,
      idPartido,
      descripcion,
      baseImponible,
      idTipoIVA,
      idCategoriaGasto,
      idEstado,
      idFormaPago,
      idProveedor,
      justificacion,
      presupuestoURL,
      facturaURL,
      contratoURL
    INTO
      r_fecha,
      r_idEjercicio,
      r_idParlamento,
      r_idParlamentario,
      r_idPartido,
      r_descripcion,
      r_baseImponible,
      r_idTipoIVA,
      r_idCategoriaGasto,
      r_idEstado,
      r_idFormaPago,
      r_idProveedor,
      r_justificacion,
      r_presupuestoURL,
      r_facturaURL,
      r_contratoURL
    FROM
      Gasto
    WHERE
      idGasto = p_idGasto;

    -- Inicializa los campos que calcula este procedimiento.
    x_idEjercicio := r_idEjercicio;
    x_idPartido := NULL;

    -- Se controlan las condiciones sobre los atributos actuales del Gasto (igual que en la baja).

    -- Controla que el Gasto no est pagado.
    IF (r_idFormaPago IS NOT NULL)
    THEN
      RAISE ErrorPagado;
    END IF;

    -- Consulta el Ejercicio al que pertenece la fecha del Gasto.
    SELECT
      inicio,
      fin,
      cerradoSN
    INTO
      x_inicio,
      x_fin,
      x_cerradoSN
    FROM
      Ejercicio
    WHERE
      inicio <= r_fecha
      AND
      r_fecha <= fin;

    -- Controla que el Ejercicio al que pertenece la fecha del Gasto no est cerrado.
    IF (x_cerradoSN = 'T')
    THEN
      RAISE ErrorEjercicioCerrado;
    END IF;

    -- Se controlan las condiciones sobre los parmetros nuevos.

    -- Controla la nueva fecha del Gasto.
    IF (p_fecha IS NULL)
    THEN
      -- Debe especificarse una fecha vlida para el Gasto.
      RAISE ErrorFecha;
    END IF;

    -- Controla si la fecha se modifica.
    IF (p_fecha <> r_fecha)
    THEN
      r_fecha := p_fecha;
      -- Controla que la nueva fecha est entre el rango de fechas de algn Ejercicio dado de alta en la base de datos.
      pkg_comun.existe_ejercicio (p_fecha);
      -- Consulta el Ejercicio al que pertenece la nueva fecha del Gasto.
      SELECT
        idEjercicio,
        inicio,
        fin,
        cerradoSN
      INTO
        x_idEjercicio,
        x_inicio,
        x_fin,
        x_cerradoSN
      FROM
        Ejercicio
      WHERE
        inicio <= p_fecha
        AND
        p_fecha <= fin;

      -- Controla si el idEjercicio se modifica.
      IF (r_idEjercicio <> x_idEjercicio)
      THEN
        -- La fecha cambia y el idEjercicio cambia.
        idEjercicioCambia := TRUE;
        -- Controla que el nuevo Ejercicio al que pertenece la nueva fecha del Gasto no est cerrado.
        IF (x_cerradoSN = 'T')
        THEN
          RAISE ErrorEjercicioCerrado;
        END IF;
      END IF;
    END IF;

    -- Controla que el nuevo Parlamento al que se imputar el Gasto exista en la base de datos.
    pkg_comun.existe_parlamento (p_idParlamento);

    -- Controla si el idParlamento se modifica.
    IF (p_idParlamento <> r_idParlamento)
    THEN
      idParlamentoCambia := TRUE;
    END IF;

    -- Controla que la nueva CategoriaGasto exista en la base de datos.
    pkg_comun.existe_categoriaGasto (p_idCategoriaGasto);
    -- Controla si idCategoriaGasto se modifica.
    IF (p_idCategoriaGasto <> r_idCategoriaGasto)
    THEN
      r_idCategoriaGasto := p_idCategoriaGasto;
      -- Consulta la nueva CategoriaGasto y su TipoGasto.
      SELECT
        CategoriaGasto.idTipoGasto,
        TipoGasto.ParlamentarioSN
      INTO
        x_idTipoGasto,
        x_ParlamentarioSN
      FROM
        CategoriaGasto,
        TipoGasto
      WHERE
        CategoriaGasto.idCategoriaGasto = p_idCategoriaGasto
        AND
        TipoGasto.idTipoGasto = CategoriaGasto.idTipoGasto;

      -- Controla si la nueva CategoriaGasto y su TipoGasto requieren que el Gasto sea asignado a un Parlamentario.
      IF (x_ParlamentarioSN = 'T')
      THEN
        -- La nueva CategoriaGasto requiere que el Gasto sea asignado a un Parlamentario.
        IF (p_idParlamentario IS NULL)
        THEN
          RAISE ErrorParlamentario;
        ELSE
          -- Controla que el Parlamentario exista en la base de datos.
          pkg_comun.existe_parlamentario (p_idParlamentario);
          -- Controla que el nuevo Parlamentario estuviera en este Parlamento en la fecha en que se produjo el Gasto.
          pkg_comun.parlamentario_en_parlamento (p_idParlamento, p_idParlamentario, p_fecha);
          -- Comprueba que el nuevo Parlamentario estuviera en algn Partido en la fecha en que se produjo el Gasto.
          -- Se considera que para poder imputar un Gasto a un Parlamentario en un Parlamento este debe estar en un Partido.
          -- Si no milita en ningn Partido entonces ha de pertenecer a un Partido ficticio llamado GRUPO MIXTO.
          pkg_comun.parlamentario_en_partido (p_idParlamentario, p_fecha);
          -- Consulta ese perodo (PartidoDH) para obtener el identificador del Partido.
          SELECT
            idPartido
          INTO
            x_idPartido
          FROM
            PartidoDH
          WHERE
            idParlamentario = p_idParlamentario
            AND
            desde <= p_fecha
            AND
            (p_fecha <= hasta
              OR
              hasta IS NULL);
        END IF;
      ELSE
        -- La CategoriaGasto no permite que el Gasto sea asignado a un Parlamentario.
        IF (p_idParlamentario IS NOT NULL)
        THEN
          RAISE ErrorNoParlamentario;
        END IF;
      END IF;
    END IF;

    -- Controla si idParlamentario se modifica.
    IF (p_idParlamentario IS NULL)
    THEN
      IF (r_idParlamentario IS NOT NULL)
      THEN
        idParlamentarioCambia := TRUE;
      END IF;
    ELSE
      IF (r_idParlamentario IS NOT NULL
          AND
          p_idParlamentario <> r_idParlamentario)
      THEN
        idParlamentarioCambia := TRUE;
      END IF;
    END IF;

    -- Controla si idPartido se modifica.
    IF (x_idPartido IS NULL)
    THEN
      IF (r_idPartido IS NOT NULL)
      THEN
        r_idPartido := x_idPartido;
      END IF;
    ELSE
      IF (r_idPartido IS NOT NULL
          AND
          x_idPartido <> r_idPartido)
      THEN
        r_idPartido := x_idPartido;
      END IF;
    END IF;

    -- Controla la nueva baseImponible del Gasto.
    IF (p_baseImponible IS NULL)
    THEN
      RAISE ErrorBaseImponible;
    ELSE
      IF (p_baseImponible < 0.00)
      THEN
        RAISE ErrorBaseImponibleNegativa;
      END IF;
    END IF;

    -- Controla si baseImponible se modifica.
    IF (p_baseImponible <> r_baseImponible)
    THEN
      r_baseImponible := p_baseImponible;
    END IF;

    -- Controla que el nuevo TipoIVA exista en la base de datos.
    pkg_comun.existe_tipoIVA (p_idTipoIVA);

    -- Controla si idTipoIVA se modifica.
    IF (p_idTipoIVA <> r_idTipoIVA)
    THEN
      r_idTipoIVA := p_idTipoIVA;
    END IF;

    -- Controla si el importe del Gasto se ha modificado.
    IF (r_baseImponible <> p_baseImponible)
    THEN
      importeGastoCambia := TRUE;
    END IF;

    -- Controla el Proveedor.
    IF (p_idProveedor IS NULL)
    THEN
      IF (p_baseImponible >= RequiereProveedor)
      THEN
        -- Se considera que el Proveedor es OBLIGATORIO si la base imponible del gasto es mayor o igual a RequiereProveedor.
        RAISE ErrorProveedor;
      END IF;
    ELSE
      -- Se considera que el Proveedor es OPCIONAL si la base imponible del gasto es menor que RequiereProveedor.
      -- Si se especifica Proveedor controla su existencia en la base de datos.
      pkg_comun.existe_PROVEEDOR (p_idProveedor);
    END IF;

    -- Controla si idProveedor se modifica.
    IF (p_idProveedor IS NULL)
    THEN
      IF (r_idProveedor IS NOT NULL)
      THEN
        r_idProveedor := p_idProveedor;
      END IF;
    ELSE
      IF (r_idProveedor IS NOT NULL
          AND
          p_idProveedor <> r_idProveedor)
      THEN
        r_idProveedor := p_idProveedor;
      END IF;
    END IF;

    -- Controla que el Estado del Gasto exista en la base de datos.
    pkg_comun.existe_ESTADO (p_idEstado);

    -- Controla si idEstado se modifica.
    IF (p_idEstado <> r_idEstado)
    THEN
      r_idEstado := p_idEstado;
    END IF;

    -- Consulta el Estado del Gasto.
    SELECT
      FormaPagoSN
    INTO
      x_formaPagoSN
    FROM
      Estado
    WHERE
      idEstado = p_idEstado;
    -- Controla si el Estado del Gasto requiere que se especifique una FormaPago.
    IF (x_formaPagoSN = 'T')
    THEN
      -- El Estado del Gasto requiere una FormaPago.
      IF (p_idFormaPago IS NULL)
      THEN
        RAISE ErrorFormaPago;
      ELSE
        -- Controla que la FormaPago exista en la base de datos.
        pkg_comun.existe_FORMAPAGO (p_idFormaPago);
      END IF;
    ELSE
      -- El Estado del Gasto no permite una FormaPago.
      IF (p_idFormaPago IS NOT NULL)
      THEN
        RAISE ErrorNoFormaPago;
      END IF;
    END IF;

    -- Controla si idFormaPago se modifica.
    IF (p_idFormaPago IS NULL)
    THEN
      IF (r_idFormaPago IS NOT NULL)
      THEN
        r_idFormaPago := p_idFormaPago;
      END IF;
    ELSE
      IF (r_idFormaPago IS NOT NULL
          AND
          r_idFormaPago <> p_idFormaPago)
      THEN
        r_idFormaPago := p_idFormaPago;
      END IF;
    END IF;

    -- No se realizan comprobaciones sobre los parmetros: p_descripcion, p_justificacion, p_presupuestoURL, p_facturaURL, p_contratoURL.

    r_descripcion := p_descripcion;
    r_justificacion := p_justificacion;
    r_presupuestoURL := p_presupuestoURL;
    r_facturaURL := p_facturaURL;
    r_contratoURL := p_contratoURL;

    -- Actualiza Estadstica.
    -- Si ha cambiado Ejercicio, Parlamento, Parlamentario o importeGasto.

    IF (idEjercicioCambia
        OR
        idParlamentoCambia
        OR
        idParlamentarioCambia
        OR
        importeGastoCambia)
    THEN
      -- Desacumula estadsticas antiguas.
      IF (r_idParlamentario IS NOT NULL)
      THEN
        -- Si el Gasto era imputable a un Parlamentario entonces se resta la base imponible del gasto
        -- en la tabla de estadstica ParlamentarioST que a su vez se resta en la tabla de estadstica ParlamentoST.
        estadistica_ParlamentarioST (r_idEjercicio, r_idParlamento, r_idParlamentario, -r_baseImponible);
      ELSE
        -- Si el Gasto no era imputable a un Parlamentario entonces se resta la base imponible del gasto
        -- en la tabla de estadstica ParlamentoST.
        estadistica_ParlamentoST (r_idEjercicio, r_idParlamento, -r_baseImponible);
      END IF;

      -- Acumula estadsticas nuevas.
      IF (p_idParlamentario IS NOT NULL)
      THEN
        -- Si el Gasto ahora es imputable a un Parlamentario entonces se suma la base imponible del gasto
        -- en la tabla de estadstica ParlamentarioST que a su vez se suma en la tabla de estadstica ParlamentoST.
        estadistica_ParlamentarioST (x_idEjercicio, p_idParlamento, p_idParlamentario, p_baseImponible);
      ELSE
        -- Si el Gasto ahora no es imputable a un Parlamentario entonces se suma la base imponible del gasto
        -- en la tabla de estadstica ParlamentoST.
        estadistica_ParlamentoST (x_idEjercicio, p_idParlamento, p_baseImponible);
      END IF;
    END IF;

    -- Realiza un update en la tabla Gasto.
    UPDATE Gasto SET
      fecha = r_fecha,
      idEjercicio = x_idEjercicio,
      idParlamento = p_idParlamento,
      idParlamentario = p_idParlamentario,
      idPartido = r_idPartido,
      descripcion = r_descripcion,
      baseImponible = r_baseImponible,
      idTipoIVA = r_idTipoIVA,
      idCategoriaGasto = r_idCategoriaGasto,
      idEstado = r_idEstado,
      idFormaPago = r_idFormaPago,
      idProveedor = r_idProveedor,
      justificacion = r_justificacion,
      presupuestoURL = r_presupuestoURL,
      facturaURL = r_facturaURL,
      contratoURL = r_contratoURL
    WHERE
      idGasto = p_idGasto;

    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN ErrorFecha THEN
      p_rsp := 'ERROR: Debe especificarse una fecha vlida para el gasto.';
    WHEN ErrorPagado THEN
      p_rsp := 'ERROR: El gasto [' || p_idGasto || '] no puede eliminarse porque est pagado.';
    WHEN ErrorEjercicioCerrado THEN
      p_rsp := 'ERROR: El ejercicio [' || x_idEjercicio || '] desde [' || x_inicio || '] hasta [' || x_fin || '] est cerrado.';
    WHEN ErrorParlamentario THEN
      p_rsp := 'ERROR: La categora de gasto [' || p_idCategoriaGasto || '] requiere que el gasto sea asignado a un parlamentario.';
    WHEN ErrorNoParlamentario THEN
      p_rsp := 'ERROR: La categora de gasto [' || p_idCategoriaGasto || '] no permite que el gasto sea asignado a un parlamentario.';
    WHEN ErrorBaseImponible THEN
      p_rsp := 'ERROR: Debe especificarse la base imponible del gasto.';
    WHEN ErrorBaseImponibleNegativa THEN
      p_rsp := 'ERROR: La base imponible del gasto no puede ser negativa.';
    WHEN ErrorProveedor THEN
      p_rsp := 'ERROR: Debe especificarse un proveedor obligatoriamente en los gastos superiores a [' || RequiereProveedor || '] euros.';
    WHEN ErrorFormaPago THEN
      p_rsp := 'ERROR: El estado [' || p_idEstado || '] requiere que se especifique una forma de pago en el gasto.';
    WHEN ErrorNoFormaPago THEN
      p_rsp := 'ERROR: El estado [' || p_idEstado || '] no permite que se especifique una forma de pago en el gasto.';
    WHEN pkg_comun.ErrorGastoNoExiste THEN
      p_rsp := 'ERROR: El gasto [' || p_idGasto || '] no existe.';
    WHEN pkg_comun.ErrorEjercicioNoExiste THEN
      p_rsp := 'ERROR: La fecha [' || p_fecha || '] no pertenece a ningn ejercicio.';
    WHEN pkg_comun.ErrorParlamentoNoExiste THEN
      p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
    WHEN pkg_comun.ErrorCategoriaGastoNoExiste THEN
      p_rsp := 'ERROR: La categora de gasto [' || p_idCategoriaGasto || '] no existe.';
    WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
    WHEN pkg_comun.ErrorParlamentarioNoParlamento THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no estaba en el parlamento [' || p_idParlamento || '] en fecha [' || p_fecha || '].';
    WHEN pkg_comun.ErrorParlamentarioNoPartido THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no estaba en ningn partido en fecha [' || p_fecha || '].';
    WHEN pkg_comun.ErrorProveedorNoExiste THEN
      p_rsp := 'ERROR: El proveedor [' || p_idProveedor || '] no existe.';
    WHEN pkg_comun.ErrorTipoIVANoExiste THEN
      p_rsp := 'ERROR: El tipo de IVA [' || p_idTipoIVA || '] no existe.';
    WHEN pkg_comun.ErrorEstadoNoExiste THEN
      p_rsp := 'ERROR: El estado [' || p_idEstado || '] no existe.';
    WHEN pkg_comun.ErrorFormaPagoNoExiste THEN
      p_rsp := 'ERROR: La forma de pago [' || p_idFormaPago || '] no existe.';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idGasto = [' || p_idGasto || '] + fecha = [' || p_fecha || '] + idParlamento = [' || p_idParlamento || '] + idParlamentario = [' || p_idParlamentario || '] + baseImponible = [' || p_baseImponible || '] + idTipoIVA = [' || p_idTipoIVA || '] + idCategoriaGasto = [' || p_idCategoriaGasto || '] + idEstado = [' || p_idEstado || '] + idFormaPago = [' || p_idFormaPago || '] + idProveedor = [' || p_idProveedor || '] + descripcion = [' || p_descripcion || '] + justificacion = [' || p_justificacion || '] + presupuestoURL = [' || p_presupuestoURL || '] + facturaURL = [' || p_facturaURL || '] + contratoURL = [' || p_contratoURL || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_gasto.modificacion', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END modificacion;

--
-- Procedimiento que actualiza la columna TotalGastosGenerales de la estadstica ParlamentoST.
--
PROCEDURE estadistica_ParlamentoST (
  p_idEjercicio  IN Gasto.idEjercicio % TYPE,
  p_idParlamento IN Gasto.idParlamento % TYPE,
  p_importeGasto IN Gasto.baseImponible % TYPE)
AS
  existe INTEGER;
BEGIN
  -- Puesto que este procedimiento es privado slo se puede invocar desde dentro del paquete y por tanto
  -- podemos confiar en que los parmetros vienen comprobados.
  -- Controla si ya existe la estadstica de este Ejericio y Parlamento en la tabla ParlamentoST de la base de datos
  SELECT
    COUNT (*)
  INTO
    existe
  FROM
    ParlamentoST
  WHERE
    (idEjercicio = p_idEjercicio
      AND
      idParlamento = p_idParlamento);

  IF (existe = 0)
  THEN
    -- Si no existe se crea la estadstica ParlamentoST de este Ejericio y Parlamento.
    -- Este es el primer Gasto acumulado en esta estadstica.
    -- La estadstica totalGastosGenerales coincidir con el importeGasto de este primer Gasto acumulado.
    -- La estadstica totalGastosParlamentarios ser inicializada al valor 0.00 por la base de datos.
    -- La estadstica numParlamentarios ser inicializada al valor 0 por la base de datos.
    -- La estadstica minGastoParlamentario ser inicializada al valor 0.00 por la base de datos.
    -- La estadstica maxGastoParlamentario ser inicializada al valor 0.00 por la base de datos.
    -- Se inserta la estadstica ParlamentoST.
    INSERT INTO ParlamentoST
        (idEjercicio,
         idParlamento,
         totalGastosGenerales)
      VALUES
        (p_idEjercicio,
         p_idParlamento,
         p_importeGasto);
  ELSE
    -- Si existe se actualiza la estadstica ParlamentoST de este Ejericio y Parlamento.
    UPDATE ParlamentoST SET
      totalGastosGenerales = totalGastosGenerales + p_importeGasto
    WHERE
      (idEjercicio = p_idEjercicio
        AND
        idParlamento = p_idParlamento);
  END IF;

END estadistica_ParlamentoST;

--
-- Procedimiento que actualiza las columnas TotalGastosParlamentarios, NumParlamentarios, MinGastoParlamentario y MaxGastoParlamentario de la estadstica ParlamentoST.
--
PROCEDURE estadistica_ParlamentoST (
  p_idEjercicio     IN Gasto.idEjercicio % TYPE,
  p_idParlamento    IN Gasto.idParlamento % TYPE,
  p_idParlamentario IN Gasto.idParlamentario % TYPE,
  p_importeGasto    IN Gasto.baseImponible % TYPE)
AS
  x_numParlamentarios     ParlamentoST.numParlamentarios % TYPE;
  x_minGastoParlamentrio  ParlamentoST.minGastoParlamentario % TYPE;
  x_maxGastoParlamentario ParlamentoST.maxGastoParlamentario % TYPE;
  existe                  INTEGER;
BEGIN
  -- Puesto que este procedimiento es privado slo se puede invocar desde dentro del paquete y por tanto
  -- podemos confiar en que los parmetros vienen comprobados.
  -- Controla si ya existe la estadstica de este Ejericio y Parlamento en la tabla ParlamentoST de la base de datos.
  SELECT
    COUNT (*)
  INTO
    existe
  FROM
    ParlamentoST
  WHERE
    (idEjercicio = p_idEjercicio
      AND
      idParlamento = p_idParlamento);

  IF (existe = 0)
  THEN
    -- Si no existe se crea la estadstica ParlamentoST de este Ejericio y Parlamento.
    -- Este es el primer Gasto acumulado en esta estadstica.
    -- La estadstica totalGastosParlamentarios coincidir con el importeGasto de este primer Gasto acumulado.
    -- La estadstica numParlamentarios ser igual a 1 (el Parlamentario de este primer Gasto acumulado).
    -- La estadstica del minGastoParlamentario coincidir con el importeGasto de este primer Gasto acumulado.
    -- La estadstica del maxGastoParlamentario tambin coincidir con el importeGasto de este primer Gasto acumulado.
    -- La estadstica totalGastosGenerales ser inicializada al valor 0.00 por la base de datos.
    -- Se inserta la estadstica ParlamentoST.
    INSERT INTO ParlamentoST
        (idEjercicio,
         idParlamento,
         totalGastosParlamentarios,
         numParlamentarios,
         minGastoParlamentario,
         maxGastoParlamentario)
      VALUES
        (p_idEjercicio,
         p_idParlamento,
         p_importeGasto,
         1,
         p_importeGasto,
         p_importeGasto);
  ELSE
    -- Se calcula de nuevo numParlamentarios: numero de Parlamentarios de este Parlamento en este Ejercicio que tienen totalGastos <> 0.00
    SELECT
      COUNT (*)
    INTO
      x_numParlamentarios
    FROM
      ParlamentarioST
    WHERE
      (idEjercicio = p_idEjercicio
        AND
        idParlamento = p_idParlamento
        AND
        totalGastos <> 0.00);
    -- Se busca cual es ahora el Parlamentario con el minGastoParlamentario este Ejercicio en este Parlamento.
    SELECT
      MIN (totalGastos)
    INTO
      x_minGastoParlamentrio
    FROM
      ParlamentarioST
    WHERE
      (idEjercicio = p_idEjercicio
        AND
        idParlamento = p_idParlamento);
    -- Se busca cual es ahora el Parlamentario con el maxGastoParlamentario este Ejercicio en este Parlamento.
    SELECT
      MAX (totalGastos)
    INTO
      x_maxGastoParlamentario
    FROM
      ParlamentarioST
    WHERE
      (idEjercicio = p_idEjercicio
        AND
        idParlamento = p_idParlamento);
    -- Finalmente se actualiza la estadstica ParlamentoST.
    UPDATE ParlamentoST SET
      totalGastosParlamentarios = totalGastosParlamentarios + p_importeGasto,
      numParlamentarios = x_numParlamentarios,
      minGastoParlamentario = x_minGastoParlamentrio,
      maxGastoParlamentario = x_maxGastoParlamentario
    WHERE
      (idEjercicio = p_idEjercicio
        AND
        idParlamento = p_idParlamento);
  END IF;

END estadistica_ParlamentoST;

--
-- Procedimiento que actualiza la columna TotalGastos de la estadstica ParlamentarioST.
--
PROCEDURE estadistica_ParlamentarioST (
  p_idEjercicio     IN Gasto.idEjercicio % TYPE,
  p_idParlamento    IN Gasto.idParlamento % TYPE,
  p_idParlamentario IN Gasto.idParlamentario % TYPE,
  p_importeGasto    IN Gasto.baseImponible % TYPE)
AS
  existe INTEGER;
BEGIN
  -- Puesto que este procedimiento es privado slo se puede invocar desde dentro del paquete y por tanto
  -- podemos confiar en que los parmetros vienen comprobados.
  -- Controla si ya existe la estadstica de este Ejericio, Parlamento y Parlamentario en la tabla ParlamentarioST.
  SELECT
    COUNT (*)
  INTO
    existe
  FROM
    ParlamentarioST
  WHERE
    (idEjercicio = p_idEjercicio
      AND
      idParlamento = p_idParlamento
      AND
      idParlamentario = p_idParlamentario);

  IF (existe = 0)
  THEN
    -- Si no existe se crea la estadstica ParlamentarioST de este Ejericio, Parlamento y Parlamentario.
    -- Este es el primer Gasto acumulado en esta estadstica de este Ejercicio, Parlamento y Parlamentario.
    INSERT INTO ParlamentarioST
        (idEjercicio,
         idParlamento,
         idParlamentario,
         totalGastos)
      VALUES
        (p_idEjercicio,
         p_idParlamento,
         p_idParlamentario,
         p_importeGasto);
  ELSE
    -- Si existe se actualiza la estadstica ParlamentarioST de este Ejercicio, Parlamento y Parlamentario.
    UPDATE ParlamentarioST SET
      totalGastos = totalGastos + p_importeGasto
    WHERE
      (idEjercicio = p_idEjercicio
        AND
        idParlamento = p_idParlamento
        AND
        idParlamentario = p_idParlamentario);
  END IF;
  -- Finalmente se acumula tambin el importeGasto en la tabla de estadstica ParlamentoST correspondiente.

  estadistica_ParlamentoST (p_idEjercicio, p_idParlamento, p_idParlamentario, p_importeGasto);
END estadistica_ParlamentarioST;

--
-- Procedimiento para cerrar un ejercicio.
--
PROCEDURE cerrar_ejercicio (
  p_idEjercicio IN  Gasto.idEjercicio % TYPE,
  p_rsp         OUT VARCHAR2)
AS
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el Ejercicio a cerrar exista en la base de datos.
    pkg_comun.existe_ejercicio (p_idEjercicio);
    -- Actualiza el Ejercicio
    UPDATE Ejercicio SET
      cerradoSN = 'T'
    WHERE
      idEjercicio = p_idEjercicio;
  EXCEPTION
    WHEN pkg_comun.ErrorEjercicioNoExiste THEN
      p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idEjercicio = [' || p_idEjercicio || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_gasto.cerrar_ejercicio', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END cerrar_ejercicio;

END pkg_gasto;
/
