CREATE OR REPLACE PACKAGE     PKG_LISTADO AS

/**
Dados un ejercicio, un parlamento y un parlamentario, obtiene un listado de todos los gastos asociados a dicho 
parlamentario en ese ejercicio y ese parlamento en orden descendente por el importe de cada gasto.

En cada lnea se muestran el identificador del gasto, la fecha en que se produjo, su base imponible, el porcentaje de IVA y el importe con IVA.

@param p_idEjercicio        Identificador del ejercicio.
@param p_idParlamento       Identificador del parlamento.
@param p_idParlamentario    Identificador del parlamentario.
@param p_rsp                Resultado de la ejecucin del procedimiento.
*/
PROCEDURE gastos_parlamentario (
  p_idEjercicio     IN  Gasto.idEjercicio % TYPE,
  p_idParlamento    IN  Gasto.idParlamento % TYPE,
  p_idParlamentario IN  Gasto.idParlamentario % TYPE,
  p_rsp             OUT VARCHAR2);

/**
Dados un ejercicio y un parlamento, obtiene un listado del total de gastos asociado a cada parlamentario de ese parlamento en ese ejercicio.
 
En cada lnea se muestran el nombre y apellidos de cada parlamentario, el nombre del partido al que pertenece (o del ltimo al que perteneci 
si ya no milita en ninguno), y el total de sus gastos.
  
@param p_idEjercicio			  Identificador del ejercicio.
@param p_idParlamento			  Identificador del parlamento.
@param p_rsp                Resultado de la ejecucin del procedimiento.
*/
PROCEDURE totalGastos_parlamentarios (
  p_idEjercicio  IN  Gasto.idEjercicio % TYPE,
  p_idParlamento IN  Gasto.idParlamento % TYPE,
  p_rsp          OUT VARCHAR2);

/**
Dado un ejercicio, obtiene un listado con el total de gastos de cada parlamento.

En cada lnea se muestran el nombre del parlamento, los gastos atribuibles a los parlamentarios, los gastos generales y los gastos totales de ese
parlamento, el nmero de parlamentarios, la media de gasto por parlamentario, el gasto mnimo y el gasto mximo realizados por un parlamentario de ese parlamento.

@param p_idEjercicio			  Identificador del ejercicio.
@param p_rsp                Resultado de la ejecucin del procedimiento.
*/
PROCEDURE totalGastos_parlamentos (
  p_idEjercicio IN  Gasto.idEjercicio % TYPE,
  p_rsp         OUT VARCHAR2);

/**
Dados un ejercicio y un parlamento, obtiene un listado de los gastos segmentado por las categoras de gasto.

En cada lnea se muestran la categora del gasto, la suma de la base imponible de los gastos de esa categora en ese parlamento y el importe con IVA.

@param p_idEjercicio			  Identificador del ejercicio.
@param p_idParlamento			  Identificador del parlamento.
@param p_rsp                Resultado de la ejecucin del procedimiento.
*/
PROCEDURE gastos_por_categoria (
  p_idEjercicio  IN  Gasto.idEjercicio % TYPE,
  p_idParlamento IN  Gasto.idParlamento % TYPE,
  p_rsp          OUT VARCHAR2);

/**
Dado un ejercicio y un parlamento, obtiene un listado de los parlamentarios que superan el gasto medio por parlamentario.

En cada lnea se muestran el nombre y apellidos del parlamentario, el nombre del partido al que pertenece (o del ltimo al que perteneci), el importe total de
los gastos atribuibles a dicho parlamentario en ese ejercicio en ese parlamento, la desviacin respecto a la media del gasto por parlamentario absoluta y en porcentaje.

@param p_idEjercicio			  Identificador del ejercicio.
@param p_idParlamento			  Identificador del parlamento.
@param p_rsp                Resultado de la ejecucin del procedimiento.
*/
PROCEDURE parlamentarios_gasto_superior (
  p_idEjercicio  IN  Gasto.idEjercicio % TYPE,
  p_idParlamento IN  Gasto.idParlamento % TYPE,
  p_rsp          OUT VARCHAR2);

/**
Dado un ejercicio, obtiene un listado del estado contable de todos los parlamentos.

En cada lnea se muestran el nombre del parlamento, el importe de los gastos pendientes de aprobacin diferenciando los pagados y los pendientes de pago,
y el importe de los gastos aprobados diferenciando los pagados de los pendientes de pago.

idEstado = 1 ( Pendiente de aprobacin. No pagado. )
idEstado = 2 ( Pendiente de aprobacin. Pagado. Gasto sujeto a posible devolucin. )
idEstado = 3 ( Aprobado. No pagado. )
idEstado = 4 ( Aprobado. Pagado. )

@param p_idEjercicio			  Identificador del ejercicio.
@param p_rsp                Resultado de la ejecucin del procedimiento.
*/
PROCEDURE estado_contable_parlamentos (
  p_idEjercicio IN  Gasto.idEjercicio % TYPE,
  p_rsp         OUT VARCHAR2);

END PKG_LISTADO;
/


CREATE OR REPLACE PACKAGE BODY     PKG_LISTADO AS

entrada Registro.entrada % TYPE;	-- String con los parmetros de entrada y sus valores.
salida Registro.salida % TYPE;		-- String con los parmetros de salida y sus valores.

espacio VARCHAR2(100);

--
-- Dados un ejercicio, un parlamento y un parlamentario, obtiene un listado de todos los gastos asociados a dicho 
-- parlamentario en ese ejercicio y ese parlamento en orden descendente por el importe de cada gasto.
--
PROCEDURE gastos_parlamentario (
  p_idEjercicio     IN  Gasto.idEjercicio % TYPE,
  p_idParlamento    IN  Gasto.idParlamento % TYPE,
  p_idParlamentario IN  Gasto.idParlamentario % TYPE,
  p_rsp             OUT VARCHAR2)
AS
  CURSOR C_GASTOS IS
    SELECT
      idGasto,
      fecha,
      baseImponible,
      TipoIVA.PorcentajeIVA,
      baseImponible * (1 + TipoIVA.PorcentajeIVA / 100) AS importeGasto
    FROM
      Gasto,
      TipoIVA
    WHERE
      idEjercicio = p_idEjercicio
      AND
      idParlamento = p_idParlamento
      AND
      idParlamentario = p_idParlamentario
      AND
      TipoIVA.idTipoIVA = Gasto.idTipoIVA
    ORDER BY importeGasto DESC;

  Parlamento_nombre       Parlamento.nombre % TYPE;
  Parlamentario_nombre    Parlamentario.nombre % TYPE;
  Parlamentario_apellidos Parlamentario.apellidos % TYPE;

  Gasto_id                Gasto.idGasto % TYPE;
  Gasto_fecha             Gasto.fecha % TYPE;
  Gasto_baseImponible     Gasto.baseImponible % TYPE;
  Gasto_importeGasto      Gasto.baseImponible % TYPE;
  TipoIVA_porcentaje      TipoIVA.PorcentajeIVA % TYPE;

  baseImponible           Gasto.baseImponible % TYPE;
  importeGasto            Gasto.baseImponible % TYPE;
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que p_idParlamento exista en la base de datos.
      pkg_comun.existe_parlamento (p_idParlamento);
      SELECT
        nombre
      INTO
        Parlamento_nombre
      FROM
        Parlamento
      WHERE
        idParlamento = p_idParlamento;
      -- Controla que p_idParlamentario exista en la base de datos.
      pkg_comun.existe_parlamentario (p_idParlamentario);
      SELECT
        nombre,
        apellidos
      INTO
        Parlamentario_nombre,
        Parlamentario_apellidos
      FROM
        Parlamentario
      WHERE
        idParlamentario = p_idParlamentario;
      -- Imprime cabecera del listado.
      DBMS_OUTPUT.ENABLE (BUFFER_SIZE => NULL);
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('       L I S T A D O    G A S T O S    P A R L A M E N T A R I O        ');
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('EJERCICIO     = [' || TO_CHAR (p_idEjercicio, 'FM9999') || ']');
      DBMS_OUTPUT.PUT_LINE ('PARLAMENTO    = [' || TO_CHAR (p_idParlamento, 'FM0999') || ']    [' || Parlamento_nombre || ']');
      DBMS_OUTPUT.PUT_LINE ('PARLAMENTARIO = [' || TO_CHAR (p_idParlamentario, 'FM0999999') || '] [' || Parlamentario_nombre || ' ' || Parlamentario_apellidos || ']');
      DBMS_OUTPUT.PUT_LINE ('=========================================================================');
      DBMS_OUTPUT.PUT_LINE ('GASTO      FECHA            BASE IMPONIBLE   %IVA                 IMPORTE');
      DBMS_OUTPUT.PUT_LINE ('-------------------------------------------------------------------------');
      OPEN C_GASTOS;
      LOOP
        FETCH C_GASTOS
        INTO Gasto_id,
        Gasto_fecha,
        Gasto_baseImponible,
        TipoIVA_porcentaje,
        Gasto_importeGasto;
        EXIT
        WHEN C_GASTOS % NOTFOUND;
        -- Imprime cada lnea.
        DBMS_OUTPUT.PUT_LINE (TO_CHAR (Gasto_id, 'FM09999999') || ' ' || TO_CHAR (Gasto_fecha, 'DD-MM-YYYY') || ' ' || TO_CHAR (Gasto_baseImponible, '99G999G990D00U') || ' ' || TO_CHAR (TipoIVA_porcentaje, '90D00') || '% ' || TO_CHAR (Gasto_importeGasto, '99G999G990D00U'));
      END LOOP;
      CLOSE C_GASTOS;
      DBMS_OUTPUT.PUT_LINE ('=========================================================================');
      SELECT
        SUM (baseImponible),
        SUM (baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
      INTO
        baseImponible,
        importeGasto
      FROM
        Gasto,
        TipoIVA
      WHERE
        idEjercicio = p_idEjercicio
        AND
        idParlamento = p_idParlamento
        AND
        idParlamentario = p_idParlamentario
        AND
        TipoIVA.idTipoIVA = Gasto.idTipoIVA;
      -- Imprime totales.
      DBMS_OUTPUT.PUT_LINE ('                    ' || TO_CHAR (baseImponible, '99G999G990D00U') || '         ' || TO_CHAR (importeGasto, '99G999G990D00U'));
      DBMS_OUTPUT.NEW_LINE;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorParlamentoNoExiste THEN
        p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
      WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
        p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || '] + idParlamento = [' || p_idParlamento || '] + idParlamentario = [' || p_idParlamentario || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_listado.gastos_parlamentario', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END gastos_parlamentario;

--
-- Dados un ejercicio y un parlamento, obtiene un listado del total de gastos asociado a cada parlamentario de ese parlamento en ese ejercicio.
--
PROCEDURE totalGastos_parlamentarios (
  p_idEjercicio  IN  Gasto.idEjercicio % TYPE,
  p_idParlamento IN  Gasto.idParlamento % TYPE,
  p_rsp          OUT VARCHAR2)
AS
  CURSOR C_TOTALGASTOS IS
    SELECT
      ParlamentarioST.idParlamentario,
      ParlamentarioST.totalGastos,
      Parlamentario.nombre,
      Parlamentario.apellidos,
      Partido.idPartido,
      Partido.nombre
    FROM
      ParlamentarioST,
      Parlamentario,
      Partido
    WHERE
      (
      ParlamentarioST.idEjercicio = p_idEjercicio
        AND
        ParlamentarioST.idParlamento = p_idParlamento
        )
      AND
      (
      Parlamentario.idParlamentario = ParlamentarioST.idParlamentario
      )
      AND
      (
      Partido.idPartido =
      (SELECT PartidoDH.idPartido
          FROM PartidoDH
        WHERE (
          PartidoDH.idParlamentario = ParlamentarioST.idParlamentario
          AND (
          (
          -- El parlamentario pertenece a un partido en la actualidad.
          hasta IS NULL       
          )
          OR
          (
          -- Cogemos el ltimo partido al que perteneci el parlamentario.
          hasta =
          (SELECT MAX (hasta)
              FROM PartidoDH
            WHERE PartidoDH.idParlamentario = ParlamentarioST.idParlamentario))     
          )
          )))
    ORDER BY ParlamentarioST.totalGastos DESC;
  
  Parlamento_nombre         Parlamento.nombre % TYPE;
  Parlamentario_id          Parlamentario.idParlamentario % TYPE;
  Parlamentario_nombre      Parlamentario.nombre % TYPE;
  Parlamentario_apellidos   Parlamentario.apellidos % TYPE;
  Parlamentario_totalGastos ParlamentarioST.totalGastos % TYPE;
  Partido_id                Partido.idPartido % TYPE;
  Partido_nombre            Partido.nombre % TYPE;
  totalGastos               ParlamentarioST.totalGastos % TYPE;
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que p_idParlamento exista en la base de datos.
      pkg_comun.existe_parlamento (p_idParlamento);
      SELECT
        nombre
      INTO
        Parlamento_nombre
      FROM
        Parlamento
      WHERE
        idParlamento = p_idParlamento;
      -- Imprime cabecera del listado.
      DBMS_OUTPUT.ENABLE (BUFFER_SIZE => NULL);
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('         L I S T A D O    T O T A L    G A S T O S    P A R L A M E N T A R I O S         ');
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('EJERCICIO     = [' || TO_CHAR (p_idEjercicio, 'FM9999') || ']');
      DBMS_OUTPUT.PUT_LINE ('PARLAMENTO    = [' || TO_CHAR (p_idParlamento, 'FM0999') || '] [' || Parlamento_nombre || ']');
      DBMS_OUTPUT.PUT_LINE ('==========================================================================================');
      DBMS_OUTPUT.PUT_LINE ('PARLAMENTARIO                          PARTIDO                                TOTAL GASTOS');
      DBMS_OUTPUT.PUT_LINE ('------------------------------------------------------------------------------------------');
      OPEN C_TOTALGASTOS;
      LOOP
        FETCH C_TOTALGASTOS
        INTO Parlamentario_id,
        Parlamentario_totalGastos,
        Parlamentario_nombre,
        Parlamentario_apellidos,
        Partido_id,
        Partido_nombre;
        EXIT
        WHEN C_TOTALGASTOS % NOTFOUND;
        -- Imprime las lneas.
        DBMS_OUTPUT.PUT_LINE (TO_CHAR (Parlamentario_id, 'FM0999999') || ' ' || RPAD (Parlamentario_nombre || ' ' || Parlamentario_apellidos, 30, ' ') || ' ' || TO_CHAR (Partido_id, 'FM0999999') || ' ' || RPAD (Partido_nombre, 20, ' ') || ' ' || TO_CHAR (Parlamentario_totalGastos, '99G999G990D00U'));
      END LOOP;
      CLOSE C_TOTALGASTOS;
      DBMS_OUTPUT.PUT_LINE ('==========================================================================================');
      SELECT
        SUM (ParlamentarioST.totalGastos)
      INTO
        totalGastos
      FROM
        ParlamentarioST
      WHERE
        ParlamentarioST.idEjercicio = p_idEjercicio
        AND
        ParlamentarioST.idParlamento = p_idParlamento;
      -- Imprime totales.
      DBMS_OUTPUT.PUT_LINE ('                                                                    ' || TO_CHAR (totalGastos, '99G999G990D00U'));
      DBMS_OUTPUT.NEW_LINE;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorParlamentoNoExiste THEN
        p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || '] + idParlamento = [' || p_idParlamento || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.totalGastos_parlamentarios', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END totalGastos_parlamentarios;

--
-- Dado un ejercicio, obtiene un listado con el total de gastos de cada parlamento.
--
PROCEDURE totalGastos_parlamentos (
  p_idEjercicio IN  Gasto.idEjercicio % TYPE,
  p_rsp         OUT VARCHAR2)
AS
  CURSOR C_TOTALGASTOS IS
    SELECT
      ParlamentoST.idParlamento,
      ParlamentoST.totalGastosParlamentarios,
      ParlamentoST.totalGastosGenerales,
      ParlamentoST.totalGastos,
      ParlamentoST.numParlamentarios,
      ParlamentoST.mediaGastoParlamentario,
      ParlamentoST.minGastoParlamentario,
      ParlamentoST.maxGastoParlamentario,
      Parlamento.nombre
    FROM
      ParlamentoST,
      Parlamento
    WHERE
      (
      ParlamentoST.idEjercicio = p_idEjercicio
      )
      AND
      (
      Parlamento.idParlamento = ParlamentoST.idParlamento
      )
    ORDER BY ParlamentoST.idParlamento;
  
  Parlamento_id                  ParlamentoST.idParlamento % TYPE;
  Parlamento_nombre              Parlamento.nombre % TYPE;
  Parlamento_totalParlamentarios ParlamentoST.totalGastosParlamentarios % TYPE;
  Parlamento_totalGenerales      ParlamentoST.totalGastosGenerales % TYPE;
  Parlamento_totalGastos         ParlamentoST.totalGastos % TYPE;
  Parlamento_numParlamentarios   ParlamentoST.numParlamentarios % TYPE;
  Parlamento_mediaParlamentario  ParlamentoST.mediaGastoParlamentario % TYPE;
  Parlamento_minParlamentario    ParlamentoST.minGastoParlamentario % TYPE;
  Parlamento_maxParlamentario    ParlamentoST.maxGastoParlamentario % TYPE;
  
  totalGastosParlamentarios      ParlamentoST.totalGastosParlamentarios % TYPE;
  totalGastosGenerales           ParlamentoST.totalGastosGenerales % TYPE;
  totalGastos                    ParlamentoST.totalGastos % TYPE;
  numParlamentarios              ParlamentoST.numParlamentarios % TYPE;
  mediaGastoParlamentario        ParlamentoST.mediaGastoParlamentario % TYPE;
  minGastoParlamentario          ParlamentoST.minGastoParlamentario % TYPE;
  maxGastoParlamentario          ParlamentoST.maxGastoParlamentario % TYPE;

BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Imprime cabecera del listado.
      DBMS_OUTPUT.ENABLE (BUFFER_SIZE => NULL);
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('                                               L I S T A D O    T O T A L    G A S T O S    P A R L A M E N T O S                                                          ');
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('EJERCICIO     = [' || TO_CHAR (p_idEjercicio, 'FM9999') || ']');
      DBMS_OUTPUT.PUT_LINE ('===========================================================================================================================================================================');
      DBMS_OUTPUT.PUT_LINE ('                                                                       G A S T O S                                        P A R L A M E N T A R I O S                      ');
      DBMS_OUTPUT.PUT_LINE ('                                            ------------------------------------------------------------  -----------------------------------------------------------------');
      DBMS_OUTPUT.PUT_LINE ('PARLAMENTO                                  PARLAMENTARIOS              GENERALES                TOTALES  NUMERO       GASTO MINIMO         GASTO MEDIO        GASTO MAXIMO');
      DBMS_OUTPUT.PUT_LINE ('---------------------------------------------------------------------------------------------------------------------------------------------------------------------------');
      OPEN C_TOTALGASTOS;
      LOOP
        FETCH C_TOTALGASTOS
        INTO Parlamento_id,
        Parlamento_totalParlamentarios,
        Parlamento_totalGenerales,
        Parlamento_totalGastos,
        Parlamento_numParlamentarios,
        Parlamento_mediaParlamentario,
        Parlamento_minParlamentario,
        Parlamento_maxParlamentario,
        Parlamento_nombre;
        EXIT
        WHEN C_TOTALGASTOS % NOTFOUND;
        -- Imprime las lneas.
        DBMS_OUTPUT.PUT_LINE (TO_CHAR (Parlamento_id, 'FM0999') || ' ' || RPAD (Parlamento_nombre, 30, ' ') || ' ' || TO_CHAR (Parlamento_totalParlamentarios, '99G999G990D00U') || ' ' || TO_CHAR (Parlamento_totalGenerales, '99G999G990D00U') || ' ' || TO_CHAR (Parlamento_totalGastos, '99G999G990D00U') || '   ' || TO_CHAR (Parlamento_numParlamentarios, '999') || ' ' || TO_CHAR (Parlamento_minParlamentario, '999G999D00U') || ' ' || TO_CHAR (Parlamento_mediaParlamentario, '999G999D00U') || ' ' || TO_CHAR (Parlamento_maxParlamentario, '999G999D00U'));
      END LOOP;
      CLOSE C_TOTALGASTOS;
      DBMS_OUTPUT.PUT_LINE ('===========================================================================================================================================================================');
      SELECT
        SUM (totalGastosParlamentarios),
        SUM (totalGastosGenerales),
        SUM (totalGastos),
        SUM (numParlamentarios),
        MEDIA (SUM (totalGastosParlamentarios), SUM (numParlamentarios)),
        MIN (minGastoParlamentario),
        MAX (maxGastoParlamentario)
      INTO
        totalGastosParlamentarios,
        totalGastosGenerales,
        totalGastos,
        numParlamentarios,
        mediaGastoParlamentario,
        minGastoParlamentario,
        maxGastoParlamentario
      FROM
        ParlamentoST
      WHERE
        ParlamentoST.idEjercicio = p_idEjercicio;
      -- Imprime totales.
      DBMS_OUTPUT.PUT_LINE ('                                    ' || TO_CHAR (totalGastosParlamentarios, '99G999G990D00U') || ' ' || TO_CHAR (totalGastosGenerales, '99G999G990D00U') || ' ' || TO_CHAR (totalGastos, '99G999G990D00U') || '   ' || TO_CHAR (numParlamentarios, '999') || ' ' || TO_CHAR (minGastoParlamentario, '999G999D00U') || ' ' || TO_CHAR (mediaGastoParlamentario, '999G999D00U') || ' ' || TO_CHAR (maxGastoParlamentario, '999G999D00U'));
      DBMS_OUTPUT.NEW_LINE;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.totalGastos_Parlamentos', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END totalGastos_parlamentos;

--
-- Dados un ejercicio y un parlamento, obtiene un listado de los gastos segmentado por las categoras de gasto.
--
PROCEDURE gastos_por_categoria (
  p_idEjercicio  IN  Gasto.idEjercicio % TYPE,
  p_idParlamento IN  Gasto.idParlamento % TYPE,
  p_rsp          OUT VARCHAR2)
AS
  CURSOR C_GASTOS IS
    SELECT
      Gasto.idCategoriaGasto,
      CategoriaGasto.descripcion,
      SUM (Gasto.baseImponible),
      SUM (Gasto.baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
    FROM
      Gasto,
      CategoriaGasto,
      TipoIVA
    WHERE
      (
      idEjercicio = p_idEjercicio
        AND
        idParlamento = p_idParlamento
        )
      AND
      (
      CategoriaGasto.idCategoriaGasto = Gasto.idCategoriaGasto
      )
      AND
      (
      TipoIVA.idTipoIVA = Gasto.idTipoIVA
      )
    GROUP BY Gasto.idCategoriaGasto, CategoriaGasto.descripcion;

  Parlamento_nombre          Parlamento.nombre % TYPE;
  Parlamentario_nombre       Parlamentario.nombre % TYPE;

  Gasto_idCategoriaGasto     Gasto.idCategoriaGasto % TYPE;
  Gasto_baseImponible        Gasto.baseImponible % TYPE;
  Gasto_importeGasto         Gasto.baseImponible % TYPE;

  CategoriaGasto_descripcion CategoriaGasto.descripcion % TYPE;
  TipoIVA_porcentaje         TipoIVA.PorcentajeIVA % TYPE;

  baseImponible              Gasto.baseImponible % TYPE;
  importeGasto               Gasto.baseImponible % TYPE;

BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que p_idParlamento exista en la base de datos.
      pkg_comun.existe_parlamento (p_idParlamento);
      SELECT
        nombre
      INTO
        Parlamento_nombre
      FROM
        Parlamento
      WHERE
        idParlamento = p_idParlamento;
      -- Imprime cabecera del listado.
      DBMS_OUTPUT.ENABLE (BUFFER_SIZE => NULL);
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('                   L I S T A D O    G A S T O S    P O R    C A T E G O R I A S                     ');
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('Ejercicio     = [' || TO_CHAR (p_idEjercicio, 'FM9999') || ']');
      DBMS_OUTPUT.PUT_LINE ('Parlamento    = [' || TO_CHAR (p_idParlamento, 'FM0999') || ']    [' || Parlamento_nombre || ']');
      DBMS_OUTPUT.PUT_LINE ('====================================================================================================');
      DBMS_OUTPUT.PUT_LINE ('CATEGORIA                                                       BASE IMPONIBLE               IMPORTE');
      DBMS_OUTPUT.PUT_LINE ('----------------------------------------------------------------------------------------------------');
      OPEN C_GASTOS;
      LOOP
        FETCH C_GASTOS
        INTO Gasto_idCategoriaGasto,
        CategoriaGasto_descripcion,
        Gasto_baseImponible,
        Gasto_importeGasto;
        EXIT
        WHEN C_GASTOS % NOTFOUND;
        DBMS_OUTPUT.PUT_LINE (TO_CHAR (Gasto_idCategoriaGasto, 'FM0999') || ' ' || RPAD (CategoriaGasto_descripcion, 50, ' ') || ' ' || TO_CHAR (Gasto_baseImponible, '99G999G990D00U') || TO_CHAR (Gasto_importeGasto, '99G999G990D00U'));
      END LOOP;
      CLOSE C_GASTOS;
      DBMS_OUTPUT.PUT_LINE ('====================================================================================================');
      SELECT
        SUM (baseImponible),
        SUM (baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
      INTO
        baseImponible,
        importeGasto
      FROM
        Gasto,
        TipoIVA
      WHERE
        (
        idEjercicio = p_idejercicio
          AND
          idParlamento = p_idParlamento
          )
        AND
        (
        TipoIVA.idTipoIVA = Gasto.idTipoIVA
        );
      -- Imprime totales.
      DBMS_OUTPUT.PUT_LINE ('                                                        ' || TO_CHAR (baseImponible, '99G999G990D00U') || TO_CHAR (importeGasto, '99G999G990D00U'));
      DBMS_OUTPUT.NEW_LINE;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorParlamentoNoExiste THEN
        p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || '] + idParlamento = [' || p_idParlamento || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_listado.gastos_por_categoria', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END gastos_por_categoria;

--
-- Dado un ejercicio y un parlamento, obtiene un listado de los parlamentarios que superan el gasto medio por parlamentario.
--
PROCEDURE parlamentarios_gasto_superior (
  p_idEjercicio  IN  Gasto.idEjercicio % TYPE,
  p_idParlamento IN  Gasto.idParlamento % TYPE,
  p_rsp          OUT VARCHAR2)
AS
  CURSOR C_PARLAMENTARIOS IS
    SELECT
      ParlamentarioST.idParlamentario,
      ParlamentarioST.totalGastos,
      Parlamentario.nombre,
      Parlamentario.apellidos,
      Partido.idPartido,
      Partido.nombre,
      (ParlamentarioST.totalGastos - ParlamentoST.mediaGastoParlamentario),
      ((ParlamentarioST.totalGastos - ParlamentoST.mediaGastoParlamentario) / ParlamentoST.mediaGastoParlamentario * 100.00)
    FROM
      ParlamentarioST,
      ParlamentoST,
      Parlamentario,
      Partido
    WHERE
      (
      ParlamentarioST.idEjercicio = p_idEjercicio
        AND
        ParlamentarioST.idParlamento = p_idParlamento
        )
      AND
      (
      ParlamentoST.idEjercicio = p_idEjercicio
        AND
        ParlamentoST.idParlamento = p_idParlamento
        )
      AND
      (
      ParlamentarioST.totalGastos > ParlamentoST.mediaGastoParlamentario
      )
      AND
      (
      Parlamentario.idParlamentario = ParlamentarioST.idParlamentario
      )
      AND
      (
      Partido.idPartido =
      (SELECT PartidoDH.idPartido
          FROM PartidoDH
        WHERE (
          PartidoDH.idParlamentario = ParlamentarioST.idParlamentario
          AND (
          (
          -- El parlamentario pertenece a un Partido en la actualidad.
          hasta IS NULL
          )
          OR
          (
          -- Cogemos el ltimo Partido al que perteneci el parlamentario.
          hasta =
          (SELECT MAX (hasta)
              FROM PartidoDH
            WHERE PartidoDH.idParlamentario = ParlamentarioST.idParlamentario))
          )
          )));

  Parlamento_nombre         Parlamento.nombre % TYPE;
  Parlamento_mediagasto     ParlamentoST.mediaGastoParlamentario % TYPE;

  Parlamentario_id          Parlamentario.idParlamentario % TYPE;
  Parlamentario_nombre      Parlamentario.nombre % TYPE;
  Parlamentario_apellidos   Parlamentario.apellidos % TYPE;
  Parlamentario_totalGastos ParlamentarioST.totalGastos % TYPE;
  Parlamentario_desviacion  ParlamentarioST.totalGastos % TYPE;
  Parlamentario_porcentaje  ParlamentarioST.totalGastos % TYPE;

  Partido_id                Partido.idPartido % TYPE;
  Partido_nombre            Partido.nombre % TYPE;

  totalGastos               ParlamentarioST.totalGastos % TYPE;
  desviacion                ParlamentarioST.totalGastos % TYPE;
  porcentaje                ParlamentarioST.totalGastos % TYPE;

BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que p_idParlamento exista en la base de datos.
      pkg_comun.existe_parlamento (p_idParlamento);
      -- Consulta el parlamento.
      SELECT
        nombre
      INTO
        Parlamento_nombre
      FROM
        Parlamento
      WHERE
        idParlamento = p_idParlamento;
      -- Consulta la estadstica ParlamentoST de este Ejercicio y Parlamento.
      SELECT
        mediaGastoParlamentario
      INTO
        Parlamento_mediagasto
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio
        AND
        idParlamento = p_idParlamento;
      -- Imprime cabecera del listado
      DBMS_OUTPUT.ENABLE (BUFFER_SIZE => NULL);
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('                L I S T A D O    P A R L A M E N T A R I O S    G A S T O    S U P E R I O R    M E D I A                    ');
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('EJERCICIO     = [' || TO_CHAR (p_idEjercicio, 'FM9999') || ']');
      DBMS_OUTPUT.PUT_LINE ('PARLAMENTO    = [' || TO_CHAR (p_idParlamento, 'FM0999') || '] [' || Parlamento_nombre || ']');
      DBMS_OUTPUT.PUT_LINE ('MEDIA GASTO   = [' || TO_CHAR (Parlamento_mediagasto, 'FM99G999G990D00U') || ']');
      DBMS_OUTPUT.PUT_LINE ('=============================================================================================================================');
      DBMS_OUTPUT.PUT_LINE ('PARLAMENTARIO                          PARTIDO                                TOTAL GASTOS             DESVIACION  PORCENTAJE');
      DBMS_OUTPUT.PUT_LINE ('-----------------------------------------------------------------------------------------------------------------------------');
      OPEN C_PARLAMENTARIOS;
      LOOP
        FETCH C_PARLAMENTARIOS
        INTO Parlamentario_id,
        Parlamentario_totalGastos,
        Parlamentario_nombre,
        Parlamentario_apellidos,
        Partido_id,
        Partido_nombre,
        Parlamentario_desviacion,
        Parlamentario_porcentaje;
        EXIT
        WHEN C_PARLAMENTARIOS % NOTFOUND;
        -- Imprime las lneas.
        DBMS_OUTPUT.PUT_LINE (TO_CHAR (Parlamentario_id, 'FM0999999') || ' ' || RPAD (Parlamentario_nombre || ' ' || Parlamentario_apellidos, 30, ' ') || ' ' || TO_CHAR (Partido_id, 'FM0999999') || ' ' || RPAD (Partido_nombre, 20, ' ') || ' ' || TO_CHAR (Parlamentario_totalGastos, '99G999G990D00U') || ' ' || TO_CHAR (Parlamentario_desviacion, '99G999G990D00U') || ' ' || TO_CHAR (Parlamentario_porcentaje, '99G990D00') || '%');
      END LOOP;
      CLOSE C_PARLAMENTARIOS;
      DBMS_OUTPUT.PUT_LINE ('=============================================================================================================================');
      SELECT
        SUM (ParlamentarioST.totalGastos),
        SUM ((ParlamentarioST.totalGastos - ParlamentoST.mediaGastoParlamentario))
      INTO
        totalGastos,
        desviacion
      FROM
        ParlamentarioST,
        ParlamentoST
      WHERE
        (
        ParlamentarioST.idEjercicio = p_idejercicio
          AND
          ParlamentarioST.idParlamento = p_idParlamento
          )
        AND
        (
        ParlamentoST.idEjercicio = p_idejercicio
          AND
          ParlamentoST.idParlamento = p_idParlamento
          )
        AND
        (
        ParlamentarioST.totalGastos > ParlamentoST.mediaGastoParlamentario
        );
      -- Imprime totales.
      DBMS_OUTPUT.PUT_LINE ('                                                                    ' || TO_CHAR (totalGastos, '99G999G990D00U') || ' ' || TO_CHAR (desviacion, '99G999G990D00U'));
      DBMS_OUTPUT.NEW_LINE;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorParlamentoNoExiste THEN
        p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || '] + idParlamento = [' || p_idParlamento || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.parlamentarios_gasto_superior', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END parlamentarios_gasto_superior;

--
-- Dado un ejercicio, obtiene un listado del estado contable de todos los parlamentos.
--
-- idEstado = 1 ( Pendiente de aprobacin. No pagado. )
-- idEstado = 2 ( Pendiente de aprobacin. Pagado. Gasto sujeto a posible devolucin. )
-- idEstado = 3 ( Aprobado. No pagado. )
-- idEstado = 4 ( Aprobado. Pagado. )
--
PROCEDURE estado_contable_parlamentos (
  p_idEjercicio IN  Gasto.idEjercicio % TYPE,
  p_rsp         OUT VARCHAR2)
AS
  CURSOR C_PARLAMENTOS IS
    SELECT
      ParlamentoST.idParlamento,
      Parlamento.nombre,
      ParlamentoST.numParlamentarios,
      (SELECT SUM (Gasto.baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
          FROM Gasto, TipoIVA
        WHERE Gasto.idEjercicio = p_idEjercicio
          AND Gasto.idParlamento = ParlamentoST.idParlamento
          AND TipoIVA.idTipoIVA = Gasto.idTipoIVA
          AND Gasto.idEstado = 1),
      (SELECT SUM (Gasto.baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
          FROM Gasto, TipoIVA
        WHERE Gasto.idEjercicio = p_idEjercicio
          AND Gasto.idParlamento = ParlamentoST.idParlamento
          AND TipoIVA.idTipoIVA = Gasto.idTipoIVA
          AND Gasto.idEstado = 2),
      (SELECT SUM (Gasto.baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
          FROM Gasto, TipoIVA
        WHERE Gasto.idEjercicio = p_idEjercicio
          AND Gasto.idParlamento = ParlamentoST.idParlamento
          AND TipoIVA.idTipoIVA = Gasto.idTipoIVA
          AND Gasto.idEstado = 3),
      (SELECT SUM (Gasto.baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
          FROM Gasto, TipoIVA
        WHERE Gasto.idEjercicio = p_idEjercicio
          AND Gasto.idParlamento = ParlamentoST.idParlamento
          AND TipoIVA.idTipoIVA = Gasto.idTipoIVA
          AND Gasto.idEstado = 4)
    FROM
      ParlamentoST,
      Parlamento
    WHERE
      ParlamentoST.idEjercicio = p_idEjercicio
      AND
      Parlamento.idParlamento = ParlamentoST.idParlamento;

  Parlamento_id                 ParlamentoST.idParlamento % TYPE;
  Parlamento_nombre             Parlamento.nombre % TYPE;
  Parlamento_numParlamentarios  ParlamentoST.numParlamentarios % TYPE;
  Parlamento_noAprobadoNoPagado ParlamentoST.totalGastos % TYPE;
  Parlamento_noAprobadoPagado   ParlamentoST.totalGastos % TYPE;
  Parlamento_AprobadoNoPagado   ParlamentoST.totalGastos % TYPE;
  Parlamento_aprobadoPagado     ParlamentoST.totalGastos % TYPE;

  noAprobadoNoPagado            ParlamentoST.totalGastos % TYPE;
  noAprobadoPagado              ParlamentoST.totalGastos % TYPE;
  aprobadoNoPagado              ParlamentoST.totalGastos % TYPE;
  aprobadoPagado                ParlamentoST.totalGastos % TYPE;

BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Imprime cabecera del listado.
      DBMS_OUTPUT.ENABLE (BUFFER_SIZE => NULL);
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('                            L I S T A D O    E S T A D O    C O N T A B L E    P A R L A M E N T O S                            ');
      DBMS_OUTPUT.NEW_LINE;
      DBMS_OUTPUT.PUT_LINE ('EJERCICIO     = [' || TO_CHAR (p_idEjercicio, 'FM9999') || ']');
      DBMS_OUTPUT.PUT_LINE ('================================================================================================================================');
      DBMS_OUTPUT.PUT_LINE ('                                                GASTOS PENDIENTES DE APROBACIN                    GASTOS APROBADOS             ');
      DBMS_OUTPUT.PUT_LINE ('                                          ------------------------------------------  ------------------------------------------');
      DBMS_OUTPUT.PUT_LINE ('PARLAMENTO                                          NO PAGADOS               PAGADOS            NO PAGADOS               PAGADOS');
      DBMS_OUTPUT.PUT_LINE ('--------------------------------------------------------------------------------------------------------------------------------');
      OPEN C_PARLAMENTOS;
      LOOP
        FETCH C_PARLAMENTOS
        INTO Parlamento_id,
        Parlamento_nombre,
        Parlamento_numParlamentarios,
        Parlamento_noAprobadoNoPagado,
        Parlamento_noAprobadoPagado,
        Parlamento_AprobadoNoPagado,
        Parlamento_aprobadoPagado;
        EXIT
        WHEN C_PARLAMENTOS % NOTFOUND;

        IF (Parlamento_noAprobadoNoPagado IS NULL)
        THEN
          Parlamento_noAprobadoNoPagado := 0.0;
        END IF;

        IF (Parlamento_noAprobadoPagado IS NULL)
        THEN
          Parlamento_noAprobadoPagado := 0.0;
        END IF;

        IF (Parlamento_AprobadoNoPagado IS NULL)
        THEN
          Parlamento_AprobadoNoPagado := 0.0;
        END IF;

        IF (Parlamento_aprobadoPagado IS NULL)
        THEN
          Parlamento_aprobadoPagado := 0.0;
        END IF;
        -- Imprime las lneas.
        DBMS_OUTPUT.PUT_LINE (TO_CHAR (Parlamento_id, 'FM0999') || ' ' || RPAD (Parlamento_nombre, 30, ' ') || ' ' || TO_CHAR (Parlamento_numParlamentarios, '990') || TO_CHAR (Parlamento_noAprobadoNoPagado, '99G999G990D00U') || TO_CHAR (Parlamento_noAprobadoPagado, '99G999G990D00U') || TO_CHAR (Parlamento_AprobadoNoPagado, '99G999G990D00U') || TO_CHAR (Parlamento_aprobadoPagado, '99G999G990D00U'));
      END LOOP;
      CLOSE C_PARLAMENTOS;
      DBMS_OUTPUT.PUT_LINE ('================================================================================================================================');
      SELECT
        SUM (
        (SELECT SUM (Gasto.baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
            FROM Gasto, TipoIVA
          WHERE Gasto.idEjercicio = p_idEjercicio
            AND Gasto.idParlamento = ParlamentoST.idParlamento
            AND TipoIVA.idTipoIVA = Gasto.idTipoIVA
            AND Gasto.idEstado = 1)),
        SUM (
        (SELECT SUM (Gasto.baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
            FROM Gasto, TipoIVA
          WHERE Gasto.idEjercicio = p_idEjercicio
            AND Gasto.idParlamento = ParlamentoST.idParlamento
            AND TipoIVA.idTipoIVA = Gasto.idTipoIVA
            AND Gasto.idEstado = 2)),
        SUM (
        (SELECT SUM (Gasto.baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
            FROM Gasto, TipoIVA
          WHERE Gasto.idEjercicio = p_idEjercicio
            AND Gasto.idParlamento = ParlamentoST.idParlamento
            AND TipoIVA.idTipoIVA = Gasto.idTipoIVA
            AND Gasto.idEstado = 3)),
        SUM (
        (SELECT SUM (Gasto.baseImponible * (1 + TipoIVA.PorcentajeIVA / 100))
            FROM Gasto, TipoIVA
          WHERE Gasto.idEjercicio = p_idEjercicio
            AND Gasto.idParlamento = ParlamentoST.idParlamento
            AND TipoIVA.idTipoIVA = Gasto.idTipoIVA
            AND Gasto.idEstado = 4))
      INTO
        noAprobadoNoPagado,
        noAprobadoPagado,
        aprobadoNoPagado,
        aprobadoPagado
      FROM
        ParlamentoST
      WHERE
        ParlamentoST.idEjercicio = p_idEjercicio;

      IF (noAprobadoNoPagado IS NULL)
      THEN
        noAprobadoNoPagado := 0.0;
      END IF;

      IF (noAprobadoPagado IS NULL)
      THEN
        noAprobadoPagado := 0.0;
      END IF;

      IF (aprobadoNoPagado IS NULL)
      THEN
        aprobadoNoPagado := 0.0;
      END IF;

      IF (aprobadoPagado IS NULL)
      THEN
        aprobadoPagado := 0.0;
      END IF;
      -- Imprime totales.
      DBMS_OUTPUT.PUT_LINE ('                                        ' || TO_CHAR (noAprobadoNoPagado, '99G999G990D00U') || TO_CHAR (noAprobadoPagado, '99G999G990D00U') || TO_CHAR (aprobadoNoPagado, '99G999G990D00U') || TO_CHAR (aprobadoPagado, '99G999G990D00U'));
      DBMS_OUTPUT.NEW_LINE;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.estado_contable_parlamentos', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END estado_contable_parlamentos;

END PKG_LISTADO;
/
