CREATE OR REPLACE PACKAGE     PKG_PARLAMENTARIO AS

ErrorParlamentoDistinto EXCEPTION;
ErrorPartidoDistinto EXCEPTION;
ErrorDesdeParlamentoDH EXCEPTION;
ErrorHastaParlamentoDH EXCEPTION;
ErrorDesdePartidoDH EXCEPTION;
ErrorHastaPartidoDH EXCEPTION;
ErrorExisteGasto EXCEPTION;

/**
Procedimiento para dar de alta un parlamentario en la base de datos.

Al insertar una fila en la base de datos, el atributo idParlamentario 
se asigna automticamente mediante una secuencia.

@param p_nombre             Nombre del parlamentario.
@param p_apellidos          Apellidos del parlamentario.
@param p_rsp                Resultado de la ejecucin del procedimiento.
*/
PROCEDURE alta (
  p_nombre    IN  Parlamentario.nombre % TYPE,
  p_apellidos IN  Parlamentario.apellidos % TYPE,
  p_rsp       OUT VARCHAR2);

/**
Procedimiento para dar de baja un parlamentario en la base de datos.

@param p_idParlamentario    Identificador del parlamentario a dar de baja.
@param p_rsp                Resultado de la ejecucin del procedimiento.
*/
PROCEDURE baja (
  p_idParlamentario IN  Parlamentario.idParlamentario % TYPE,
  p_rsp             OUT VARCHAR2);

/**
Procedimiento para modificar un parlamentario en la base de datos.

@param p_idParlamentario    Identificador del parlamentario a modificar.
@param p_nombre             Nombre del parlamentario.
@param p_apellidos          Apellidos del parlamentario.
@param p_rsp                Resultado de la ejecucin del procedimiento.
*/
PROCEDURE modificacion (
  p_idParlamentario IN  Parlamentario.idParlamentario % TYPE,
  p_nombre          IN  Parlamentario.nombre % TYPE,
  p_apellidos       IN  Parlamentario.apellidos % TYPE,
  p_rsp             OUT VARCHAR2);

/**
Procedimiento para asignar un parlamentario a un parlamento.

@param p_idParlamentario          Identificador del parlamentario.
@param p_idParlamento             Identificador del parlamento.
@param p_desde                    Fecha de inicio del perodo del parlamentario en el parlamento.
@param p_rsp                      Resultado de la ejecucin del procedimiento.

@throws ErrorDesdeParlamentoDH    Si la fecha de inicio del perodo es anterior a la fecha de finalizacin del ltimo perodo.
*/
PROCEDURE parlamento_desde (
  p_idParlamentario IN  ParlamentoDH.idParlamentario % TYPE,
  p_idParlamento    IN  ParlamentoDH.idParlamento % TYPE,
  p_desde           IN  ParlamentoDH.desde % TYPE,
  p_rsp             OUT VARCHAR2);

/**
Procedimiento para desvincular a un parlamentario de un parlamento.

@param p_idParlamentario          Identificador del parlamentario.
@param p_idParlamento             Identificador del parlamento.
@param p_hasta                    Fecha de finalizacin del perodo del parlamentario en el parlamento.
@param p_rsp                      Resultado de la ejecucin del procedimiento.

@throws ErrorParlamentoDistinto   Si el parlamentario ya estaba vinculado a un parlamento distinto al especificado.
@throws ErrorHastaParlamentoDH    Si la fecha de final del perodo es anterior a la fecha de inicio del mismo.
@throws ErrorExisteGasto          Si existe algn gasto imputado al parlamentario posterior a la fecha de final del perodo.
*/
PROCEDURE parlamento_hasta (
  p_idParlamentario IN  ParlamentoDH.idParlamentario % TYPE,
  p_idParlamento    IN  ParlamentoDH.idParlamento % TYPE,
  p_hasta           IN  ParlamentoDH.hasta % TYPE,
  p_rsp             OUT VARCHAR2);

/**
Procedimiento para asignar un parlamentario a un partido.

@param p_idParlamentario          Identificador del parlamentario.
@param p_idPartido                Identificador del partido.
@param p_desde                    Fecha de inicio del perodo del parlamentario en el partido.
@param p_rsp                      Resultado de la ejecucin del procedimiento.

@throws ErrorDesdePartidoDH       Si la fecha de inicio del perodo es anterior a la fecha de finalizacin del ltimo perodo.
*/
PROCEDURE partido_desde (
  p_idParlamentario IN  PartidoDH.idParlamentario % TYPE,
  p_idPartido       IN  PartidoDH.idPartido % TYPE,
  p_desde           IN  PartidoDH.desde % TYPE,
  p_rsp             OUT VARCHAR2);

/**
Procedimiento para desvincular a un parlamentario de un partido.

@param p_idParlamentario          Identificador del parlamentario.
@param p_idPartido                Identificador del partido.
@param p_hasta                    Fecha de finalizacin del perodo del parlamentario en el partido.
@param p_rsp                      Resultado de la ejecucin del procedimiento.

@throws ErrorPartidoDistinto      Si el parlamentario ya estaba vinculado a un partido distinto al especificado.
@throws ErrorHastaPartidoDH       Si la fecha de final del perodo es anterior a la fecha de inicio del mismo.
@throws ErrorExisteGasto          Si existe algn gasto imputado al parlamentario posterior a la fecha de final del perodo.
*/
PROCEDURE partido_hasta (
  p_idParlamentario IN  PartidoDH.idParlamentario % TYPE,
  p_idPartido       IN  PartidoDH.idPartido % TYPE,
  p_hasta           IN  PartidoDH.hasta % TYPE,
  p_rsp             OUT VARCHAR2);

END PKG_PARLAMENTARIO;
/


CREATE OR REPLACE PACKAGE BODY     PKG_PARLAMENTARIO AS

entrada Registro.entrada % TYPE;	-- String con los parmetros de entrada y sus valores.
salida Registro.salida % TYPE;		-- String con los parmetros de salida y sus valores.

--
-- Procedimiento para dar de alta un parlamentario en la base de datos.
--
PROCEDURE alta (
  p_nombre    IN  Parlamentario.nombre % TYPE,
  p_apellidos IN  Parlamentario.apellidos % TYPE,
  p_rsp       OUT VARCHAR2)
AS
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Inserta un Parlamentario en la base de datos usando los parmetros recibidos.
    -- El atributo idParlamentario se asigna automticamente mediante una secuencia.
    INSERT INTO Parlamentario
        (nombre,
         apellidos)
      VALUES
        (p_nombre,
         p_apellidos);
    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'nombre = [' || p_nombre || '] + apellidos = [' || p_apellidos || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamentario.alta', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END alta;

--
-- Procedimiento para dar de baja un parlamentario en la base de datos.
--
PROCEDURE baja (
  p_idParlamentario IN  Parlamentario.idParlamentario % TYPE,
  p_rsp             OUT VARCHAR2)
AS
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el parlamentario a dar de baja exista en la base de datos.
    pkg_comun.existe_parlamentario (p_idParlamentario);
    DELETE FROM
        Parlamentario
      WHERE
        idParlamentario = p_idParlamentario;
     -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idParlamentario = [' || p_idParlamentario || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamentario.baja', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END baja;

--
-- Procedimiento para modificar un parlamentario en la base de datos.
-- 
PROCEDURE modificacion (
  p_idParlamentario IN  Parlamentario.idParlamentario % TYPE,
  p_nombre          IN  Parlamentario.nombre % TYPE,
  p_apellidos       IN  Parlamentario.apellidos % TYPE,
  p_rsp             OUT VARCHAR2)
AS
  existe NUMBER;
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el parlamentario a modificar exista en la base de datos.
    pkg_comun.existe_parlamentario (p_idParlamentario);
    -- Realiza el update en la tabla Parlamentario.
    UPDATE Parlamentario SET
      nombre = p_nombre,
      apellidos = p_apellidos
    WHERE
      idParlamentario = p_idParlamentario;
    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idParlamentario = [' || p_idParlamentario || '] + nombre = [' || p_nombre || '] + apellidos = [' || p_apellidos || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamentario.modificacion', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END modificacion;

--
-- Procedimiento para asignar un parlamentario a un parlamento.
--
PROCEDURE parlamento_desde (
  p_idParlamentario IN  ParlamentoDH.idParlamentario % TYPE,
  p_idParlamento    IN  ParlamentoDH.idParlamento % TYPE,
  p_desde           IN  ParlamentoDH.desde % TYPE,
  p_rsp             OUT VARCHAR2)
AS
  existe   NUMBER;
  maxHasta ParlamentoDH.hasta % TYPE;
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el parlamentario exista en la base de datos.
    pkg_comun.existe_parlamentario (p_idParlamentario);
    -- Controla que el parlamento exista en la base de datos.
    pkg_comun.existe_Parlamento (p_idParlamento);
    -- Controla que el parlamentario no est ya en algn parlamento en la actualidad.
    pkg_comun.parlamentario_no_en_parlamento (p_idParlamentario);
    -- Controla que el perodo (ParlamentoDH) de este parlamentario en este parlamento empiece despus de finalizado cualquier perodo
    -- anterior (si existe alguno) de este parlamentario en cualquier parlamento.
    SELECT
      MAX (hasta)
    INTO
      maxHasta
    FROM
      ParlamentoDH
    WHERE
      idParlamentario = p_idParlamentario;

    IF (maxHasta IS NOT NULL
        AND
        maxHasta >= p_desde)
    THEN
      RAISE ErrorDesdeParlamentoDH;
    END IF;

    -- Si todas la comprobaciones anteriores fueron satisfactorias realiza una insercin de un nuevo perodo en la tabla ParlamentoDH
    -- correspondiente al inicio de un perodo de este parlamentario en este parlamento desde esta fecha y con
    -- fecha de finalizacin indefinida.

    INSERT INTO ParlamentoDH
        (idParlamentario,
         idParlamento,
         desde,
         hasta)
      VALUES
        (p_idParlamentario,
         p_idParlamento,
         p_desde,
         NULL);
    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
    WHEN pkg_comun.ErrorParlamentoNoExiste THEN
      p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
    WHEN pkg_comun.ErrorParlamentarioParlamento THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] ya est en un parlamento actualmente.';
    WHEN ErrorDesdeParlamentoDH THEN
      p_rsp := 'ERROR: El inicio del perodo del parlamentario [' || p_idParlamentario || '] en el parlamento [' || p_idParlamento || '] debe ser posterior a [' || maxHasta || '].';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idParlamentario = [' || p_idParlamentario || '] + idParlamento = [' || p_idParlamento || '] + desde = [' || p_desde || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamentario.parlamento_desde', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END parlamento_desde;

--
-- Procedimiento para desvincular a un parlamentario de un parlamento.
--
PROCEDURE parlamento_hasta (
  p_idParlamentario IN  ParlamentoDH.idParlamentario % TYPE,
  p_idParlamento    IN  ParlamentoDH.idParlamento % TYPE,
  p_hasta           IN  ParlamentoDH.hasta % TYPE,
  p_rsp             OUT VARCHAR2)
AS
  existe            NUMBER;
  r_idParlamentoDH  ParlamentoDH.idParlamentoDH % TYPE;
  r_idParlamentario ParlamentoDH.idParlamentario % TYPE;
  r_idParlamento    ParlamentoDH.idParlamento % TYPE;
  r_desde           ParlamentoDH.desde % TYPE;
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el parlamentario exista en la base de datos.
    pkg_comun.existe_parlamentario (p_idParlamentario);
    -- Controla que el parlamento exista en la base de datos.
    pkg_comun.existe_Parlamento (p_idParlamento);
    -- Controla que el parlamentario est en algn parlamento en la actualidad.
    pkg_comun.parlamentario_en_parlamento (p_idParlamentario);
    -- Consulta el perodo (ParlamentoDH) en curso de este parlamentario.
    SELECT
      idParlamentoDH,
      idParlamentario,
      idParlamento,
      desde
    INTO
      r_idParlamentoDH,
      r_idParlamentario,
      r_idParlamento,
      r_desde
    FROM
      ParlamentoDH
    WHERE
      idParlamentario = p_idParlamentario
      AND
      hasta IS NULL;

    -- Controla si el parlamentario est actualmente en el parlamento del que queremos cerrar el perodo.
    IF (r_idParlamento <> p_idParlamento)
    THEN
      RAISE ErrorParlamentoDistinto;
    END IF;

    -- Controla que la fecha de finalizacin del perodo sea posterior a la fecha de inicio del perodo.
    IF (r_desde > p_hasta)
    THEN
      RAISE ErrorHastaParlamentoDH;
    END IF;

    -- Controla que no exista ningn gasto imputado a este parlamentario posterior a la fecha en que se quiere 
    -- cerrar el perodo.
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      Gasto
    WHERE
      idParlamentario = p_idParlamentario
		AND
      idParlamento = p_idParlamento
      AND
      fecha > p_hasta;

    IF (existe <> 0)
    THEN
      RAISE ErrorExisteGasto;
    END IF;

    -- Si todas la comprobaciones anteriores fueron satisfactorias actualiza el perodo en la tabla ParlamentoDH
    -- para indicar la finalizacin en esta fecha del perodo en curso de este parlamentario en este parlamento.

    UPDATE ParlamentoDH SET
      hasta = p_hasta
    WHERE
      idParlamentoDH = r_idParlamentoDH;
    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
    WHEN pkg_comun.ErrorParlamentoNoExiste THEN
      p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
    WHEN pkg_comun.ErrorParlamentarioNoParlamento THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no est en ningn parlamento actualmente.';
    WHEN ErrorParlamentoDistinto THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] actualmente est en el parlamento [' || r_idParlamento || '].';
    WHEN ErrorHastaParlamentoDH THEN
      p_rsp := 'ERROR: El final del perodo del parlamentario [' || p_idParlamentario || '] en el parlamento [' || p_idParlamento || '] no puede ser anterior a [' || r_desde || '].';
    WHEN ErrorExisteGasto THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] del parlamento [' || p_idParlamento || '] tiene gastos imputados posteriores a [' || p_hasta || '].';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;  
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idParlamentario = [' || p_idParlamentario || '] + idParlamento = [' || p_idParlamento || '] + hasta = [' || p_hasta || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamentario.parlamento_hasta', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END parlamento_hasta;

--
-- Procedimiento para asignar un parlamentario a un partido.
--
PROCEDURE partido_desde (
  p_idParlamentario IN  PartidoDH.idParlamentario % TYPE,
  p_idPartido       IN  PartidoDH.idPartido % TYPE,
  p_desde           IN  PartidoDH.desde % TYPE,
  p_rsp             OUT VARCHAR2)
AS
  existe   NUMBER;
  maxHasta PartidoDH.hasta % TYPE;
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el parlamentario exista en la base de datos.
    pkg_comun.existe_parlamentario (p_idParlamentario);
    -- Controla que el partido exista en la base de datos.
    pkg_comun.existe_partido (p_idPartido);
    -- Controla que el parlamentario no est ya en algn partido en la actualidad.
    pkg_comun.parlamentario_no_en_partido (p_idParlamentario);
    -- Controla que el perodo (PartidoDH) de este parlamentario en este partido empiece despus de finalizado cualquier perodo
    -- anterior (si existe alguno) de este parlamentario en cualquier partido.
    SELECT
      MAX (hasta)
    INTO
      maxHasta
    FROM
      PartidoDH
    WHERE
      idParlamentario = p_idParlamentario;

    IF (maxHasta IS NOT NULL
        AND
        maxHasta >= p_desde)
    THEN
      RAISE ErrorDesdePartidoDH;
    END IF;

    -- Si todas la comprobaciones anteriores fueron satisfactorias realiza una insercin de un perodo en la tabla PartidoDH
    -- correspondiente al inicio de un perodo de este parlamentario en este partido desde esta fecha y con
    -- fecha de finalizacin indefinida.

    INSERT INTO PartidoDH
        (idParlamentario,
         idPartido,
         desde,
         hasta)
      VALUES
        (p_idParlamentario,
         p_idPartido,
         p_desde,
         NULL);
    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
    WHEN pkg_comun.ErrorPartidoNoExiste THEN
      p_rsp := 'ERROR: El partido [' || p_idPartido || '] no existe.';
    WHEN pkg_comun.ErrorParlamentarioPartido THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] ya est en un partido actualmente.';
    WHEN ErrorDesdePartidoDH THEN
      p_rsp := 'ERROR: El inicio del perodo del parlamentario [' || p_idParlamentario || '] en el partido [' || p_idPartido || '] debe ser posterior a [' || maxHasta || '].';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idParlamentario = [' || p_idParlamentario || '] + idPartido = [' || p_idPartido || '] + desde = [' || p_desde || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamentario.partido_desde', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END partido_desde;

--
-- Procedimiento para desvincular a un parlamentario de un partido.
--
PROCEDURE partido_hasta (
  p_idParlamentario IN  PartidoDH.idParlamentario % TYPE,
  p_idPartido       IN  PartidoDH.idPartido % TYPE,
  p_hasta           IN  PartidoDH.hasta % TYPE,
  p_rsp             OUT VARCHAR2)
AS
  existe            NUMBER;
  r_idPartidoDH     PartidoDH.idPartidoDH % TYPE;
  r_idParlamentario PartidoDH.idParlamentario % TYPE;
  r_idPartido       PartidoDH.idPartido % TYPE;
  r_desde           PartidoDH.desde % TYPE;
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el parlamentario exista en la base de datos.
    pkg_comun.existe_parlamentario (p_idParlamentario);
    -- Controla que el partido exista en la base de datos.
    pkg_comun.existe_partido (p_idPartido);
    -- Controla que el parlamentario est en algn partido en la actualidad.
    pkg_comun.parlamentario_en_partido (p_idParlamentario);
    -- Consulta el perodo (PartidoDH) en curso de este parlamentario.
    SELECT
      idPartidoDH,
      idParlamentario,
      idPartido,
      desde
    INTO
      r_idPartidoDH,
      r_idParlamentario,
      r_idPartido,
      r_desde
    FROM
      PartidoDH
    WHERE
      idParlamentario = p_idParlamentario
      AND
      hasta IS NULL;

    -- Controla si el parlamentario est actualmente en el partido del que queremos cerrar el perodo.
    IF (r_idPartido <> p_idPartido)
    THEN
      RAISE ErrorPartidoDistinto;
    END IF;

    -- Controla que la fecha de finalizacin del perodo sea posterior a la fecha de inicio del perodo.
    IF (r_desde > p_hasta)
    THEN
      RAISE ErrorHastaPartidoDH;
    END IF;

    -- Controla que no exista ningn gasto imputado a este parlamentario y partido posterior a la fecha en 
    -- que se quiere cerrar el perodo.
    SELECT
      COUNT (*)
    INTO
      existe
    FROM
      Gasto
    WHERE
      idParlamentario = p_idParlamentario
      AND
      idPartido = p_idPartido
      AND
      fecha > p_hasta;

    IF (existe <> 0)
    THEN
      RAISE ErrorExisteGasto;
    END IF;

    -- Si todas la comprobaciones anteriores fueron satisfactorias actualiza el perodo en la tabla PartidoDH
    -- para indicar la finalizacin en esta fecha del perodo de este parlamentario en este partido.

    UPDATE PartidoDH SET
      hasta = p_hasta
    WHERE
      idPartidoDH = r_idPartidoDH;
    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
    WHEN pkg_comun.ErrorPartidoNoExiste THEN
      p_rsp := 'ERROR: El partido [' || p_idPartido || '] no existe.';
    WHEN pkg_comun.ErrorParlamentarioNoPartido THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no est en ningn partido actualmente.';
    WHEN ErrorPartidoDistinto THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] actualmente est en el partido [' || r_idPartido || '].';
    WHEN ErrorHastaPartidoDH THEN
      p_rsp := 'ERROR: El final del perodo del parlamentario [' || p_idParlamentario || '] en el partido [' || p_idPartido || '] no puede ser anterior a [' || r_desde || '].';
    WHEN ErrorExisteGasto THEN
      p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] del partido [' || p_idPartido || '] tiene gastos imputados posteriores a [' || p_hasta || '].';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idParlamentario = [' || p_idParlamentario || '] + idPartido = [' || p_idPartido || '] + hasta = [' || p_hasta || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamentario.partido_hasta', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END partido_hasta;

END PKG_PARLAMENTARIO;
/
