CREATE OR REPLACE PACKAGE     pkg_parlamento AS

ErrorNumeroParlamentarios EXCEPTION;

/**
Procedimiento para dar de alta un parlamento en la base de datos.

Al insertar una fila en la base de datos, el atributo idParlamento 
se asigna automticamente mediante una secuencia.

@param p_nombre                       Nombre del parlamento.
@param p_idCiudad                     Identificador de la ciudad donde tiene la sede el parlamento.
@param p_idTipoParlamento             Identificador del tipo de parlamento.
@param p_direccion                    Direccin de la sede del parlamento.
@param p_codigoPostal                 Cdigo postal de la sede del parlamento.
@param p_telefono                     Telfono de la centralita de la sede del parlamento.
@param p_web                          URL de la web del parlamento.
@param p_maxParlamentarios            Nmero mximo de parlamentarios del parlamento.
@param p_rsp                          Resultado de la ejecucin del procedimiento.
*/
PROCEDURE alta (
  p_nombre            IN  Parlamento.nombre % TYPE,
  p_idCiudad          IN  Parlamento.idCiudad % TYPE,
  p_idTipoParlamento  IN  Parlamento.idTipoParlamento % TYPE,
  p_direccion         IN  Parlamento.direccion % TYPE,
  p_codigoPostal      IN  Parlamento.codigoPostal % TYPE,
  p_telefono          IN  Parlamento.telefono % TYPE,
  p_web               IN  Parlamento.web % TYPE,
  p_maxParlamentarios IN  Parlamento.maxParlamentarios % TYPE,
  p_rsp               OUT VARCHAR2);

/**
Procedimiento para dar de baja un parlamentario en la base de datos.

@param p_idParlamento                 Identificador del parlamento a dar de baja.
@param p_rsp                          Resultado de la ejecucin del procedimiento.
*/
PROCEDURE baja (
  p_idParlamento IN  Parlamento.idParlamento % TYPE,
  p_rsp          OUT VARCHAR2);

/**
Procedimiento para modificar un parlamento en la base de datos.

@param p_idParlamento                 Identificador del parlamento a modificar.
@param p_nombre                       Nombre del parlamento.
@param p_idCiudad                     Identificador de la ciudad donde tiene la sede el parlamento.
@param p_idTipoParlamento             Identificador del tipo de parlamento.
@param p_direccion                    Direccin de la sede del parlamento.
@param p_codigoPostal                 Cdigo postal de la sede del parlamento.
@param p_telefono                     Telfono de la centralita de la sede del parlamento.
@param p_web                          URL de la web del parlamento.
@param p_maxParlamentarios            Nmero mximo de parlamentarios del parlamento.
@param p_rsp                          Resultado de la ejecucin del procedimiento.

@throws ErrorNumeroParlamentarios     Si el nmero de parlamentarios nuevo es inferior al nmero de parlamentarios activos.
*/
PROCEDURE modificacion (
  p_idParlamento      IN  Parlamento.idParlamento % TYPE,
  p_nombre            IN  Parlamento.nombre % TYPE,
  p_idCiudad          IN  Parlamento.idCiudad % TYPE,
  p_idTipoParlamento  IN  Parlamento.idTipoParlamento % TYPE,
  p_direccion         IN  Parlamento.direccion % TYPE,
  p_codigoPostal      IN  Parlamento.codigoPostal % TYPE,
  p_telefono          IN  Parlamento.telefono % TYPE,
  p_web               IN  Parlamento.web % TYPE,
  p_maxParlamentarios IN  Parlamento.maxParlamentarios % TYPE,
  p_rsp               OUT VARCHAR2);

END pkg_parlamento;
/


CREATE OR REPLACE PACKAGE BODY     PKG_PARLAMENTO AS

entrada Registro.entrada % TYPE;	-- String con los parmetros de entrada y sus valores.
salida Registro.salida % TYPE;		-- String con los parmetros de salida y sus valores.

--
-- Procedimiento para dar de alta un parlamento en la base de datos.
--
PROCEDURE alta (
  p_nombre            IN  Parlamento.nombre % TYPE,
  p_idCiudad          IN  Parlamento.idCiudad % TYPE,
  p_idTipoParlamento  IN  Parlamento.idTipoParlamento % TYPE,
  p_direccion         IN  Parlamento.direccion % TYPE,
  p_codigoPostal      IN  Parlamento.codigoPostal % TYPE,
  p_telefono          IN  Parlamento.telefono % TYPE,
  p_web               IN  Parlamento.web % TYPE,
  p_maxParlamentarios IN  Parlamento.maxParlamentarios % TYPE,
  p_rsp               OUT VARCHAR2)
AS
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    INSERT INTO Parlamento
        (nombre,
         IdCiudad,
         IdTipoParlamento,
         direccion,
         codigoPostal,
         telefono,
         web,
         maxParlamentarios)
      VALUES
        (p_nombre,
         p_idCiudad,
         p_idTipoParlamento,
         p_direccion,
         p_codigoPostal,
         p_telefono,
         p_web,
         p_maxParlamentarios);
    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'nombre = [' || p_nombre || '] + idCiudad = [' || p_idCiudad || '] + idTipoParlamento = [' || p_idTipoParlamento || '] + direccion = [' || p_direccion || '] + codigoPostal = [' || p_codigoPostal || '] + telefono = [' || p_telefono || '] + web = [' || p_web || '] + maxParlamentarios = [' || p_maxParlamentarios || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamento.alta', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END alta;


--
-- Procedimiento para dar de baja un parlamento en la base de datos.
--
PROCEDURE baja (
  p_idParlamento IN  Parlamento.idParlamento % TYPE,
  p_rsp          OUT VARCHAR2)
AS
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el Parlamento a dar de baja exista en la base de datos
    pkg_comun.existe_parlamento (p_idParlamento);
    DELETE FROM
        Parlamento
      WHERE
        idParlamento = p_idParlamento;
    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN pkg_comun.ErrorParlamentoNoExiste THEN
      p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idParlamento = [' || p_idParlamento || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamento.baja', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END baja;


--
-- Procedimiento para modificar un parlamento en la base de datos.
-- 
PROCEDURE modificacion (
  p_idParlamento      IN  Parlamento.idParlamento % TYPE,
  p_nombre            IN  Parlamento.nombre % TYPE,
  p_idCiudad          IN  Parlamento.idCiudad % TYPE,
  p_idTipoParlamento  IN  Parlamento.idTipoParlamento % TYPE,
  p_direccion         IN  Parlamento.direccion % TYPE,
  p_codigoPostal      IN  Parlamento.codigoPostal % TYPE,
  p_telefono          IN  Parlamento.telefono % TYPE,
  p_web               IN  Parlamento.web % TYPE,
  p_maxParlamentarios IN  Parlamento.maxParlamentarios % TYPE,
  p_rsp               OUT VARCHAR2)
AS
  numParlamentariosActuales NUMBER;
BEGIN
  -- Inicializa el resultado de ejecucin del procedimiento a OK.
  p_rsp := 'OK';
  BEGIN
    -- Controla que el Parlamento a modificar exista en la base de datos
    pkg_comun.existe_parlamento (p_idParlamento);
    -- Controla que el nuevo nmero mximo de Parlamentarios p_maxParlamentarios no sea menor que el nmero de Parlamentarios activos actualmente en ese Parlamento
    SELECT
      COUNT (*)
    INTO
      numParlamentariosActuales
    FROM
      ParlamentoDH
    WHERE
      idParlamento = p_idParlamento
      AND
      hasta IS NULL;

    IF (p_maxParlamentarios < numParlamentariosActuales)
    THEN
      RAISE ErrorNumeroParlamentarios;
    END IF;

    -- Realiza el update en la tabla Parlamento
    UPDATE Parlamento SET
      nombre = p_nombre,
      IdCiudad = p_idCiudad,
      IdTipoParlamento = p_idTipoParlamento,
      direccion = p_direccion,
      codigoPostal = p_codigoPostal,
      telefono = p_telefono,
      web = p_web,
      maxParlamentarios = p_maxParlamentarios
    WHERE
      idParlamento = p_idParlamento;
    -- Si no ha habido excepciones confirma los cambios en la base de datos.
    COMMIT;
  EXCEPTION
    WHEN ErrorNumeroParlamentarios THEN
      p_rsp := 'ERROR: El nmero de parlamentarios del parlamento [' || p_idParlamento || '] no puede ser inferior a [' || numParlamentariosActuales || '] parlamentarios actuales.';
    WHEN pkg_comun.ErrorParlamentoNoExiste THEN
      p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
    WHEN OTHERS THEN
      p_rsp := 'ERROR: ' || SQLERRM;
      -- Si ha habido excepciones distintas de las anteriores deshace los posibles cambios en la base de datos.
      ROLLBACK;
  END;
  -- Construye el string con los parmetros de entrada y sus valores.
  entrada := 'idParlamento = [' || p_idParlamento || '] + nombre = [' || p_nombre || '] + idCiudad = [' || p_idCiudad || '] + idTipoParlamento = [' || p_idTipoParlamento || '] + direccion = [' || p_direccion || '] + codigoPostal = [' || p_codigoPostal || '] + telefono = [' || p_telefono || '] + web = [' || p_web || '] + maxParlamentarios = [' || p_maxParlamentarios || ']';
  -- Construye el string con los parmetros de salida y sus valores.
  salida := 'rsp = ' || p_rsp;
  -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
  pkg_comun.entrada_registro ('pkg_parlamento.modificacion', USER, SYSTIMESTAMP, entrada, salida);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
END modificacion;

END PKG_PARLAMENTO;
/
